/*
FILE:     scale.C
*/
/*
VERSION:  3.28
*/
/*
DATE:     4/15/2021
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2021 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               RCSB PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/
/* a program to gather all the reflection data and convert it to
   cif format.   March, 2002
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "rcsb_math.h"
#include "structure.h"
#include "util.h"
#include "harv.h"

const float DEG2RAD= 0.017453292;
#define NSET 20   // number of data set for phasing

extern DIFFRN_RADIATION_WAVELENGTH diffrn_radiation_wavelength;
extern DIFFRN_RADIATION diffrn_radiation;
extern REFLNS reflns;
extern REFINE refine;
extern REFLNS_SHELL reflns_shell;
extern EXPTL_CRYSTAL exptl_crystal;
extern CELL cell ;
extern SYMMETRY symmetry;
REFLNS reflns_phs[NSET];
REFLNS_SHELL reflns_shell_phs[NSET];

char DATACUT[20];


float get_resolution(float h,float k, float l, float *rcell);
void calc_recip_cell(float *cell, float *rcell);
void I_over_sigmaI(char *inpfile);
void extract_CrysAlis_log(char *inpfile); //Rigaku, not yet.
void extract_XSCALE_log(char *inpfile);
void extract_XGEN_log(char *inpfile);
void extract_PROSCALE_log(char *inpfile);
void extract_XPREP_log(char *inpfile);
void extract_MARSCALE_log( char *inpfile );
void extract_XIA2_log(char *inpfile );
void add_value2cell(float a, float b,float c,float alpha,float beta,float garma);
void write_diffrn_shell(FILE *fout, REFLNS_SHELL *log,  int i);
void write_diffrn(FILE *fout, REFLNS *log,  int i);


void extract_scale(FILE *fout, int argc, char **argv, int start, char *prog)
{
    int i=0, key=0;
    char infile[512], info[512];
    const char *id = "-i -p -e -d -m -r -s -sp  -v";

    for(i=0; i<argc-1; i++){
        if(!strcmp_case(argv[i],"-DATACUT") ){
            strcpy(DATACUT,  argv[i+1]);
        }
    }
    diffrn_radiation.diffrn_id=(char *)"1";
    reflns.pdbx_diffrn_id=(char *)"1";
    reflns.pdbx_ordinal=(char *)"1";
    for(i=0; i<NBIN; i++){reflns_shell.pdbx_diffrn_id[i]=(char *)"1";}

    for(i=start; i<argc-1; i++){
        if(strstr_case(id, argv[i])) break;
        if(!strcmp_case(argv[i],"-ILOG") ){
            strcpy(infile, argv[i+1]);
            
            if(!strcmp_case(prog, "HKL")|| !strcmp_case(prog,"SCALEPACK")||
               !strcmp_case(prog, "HKL2000")){
                
                printf("HKL/Scalepack output Log file: (%s)\n",infile);
                
                extract_HKL_log(infile);
                key=1;
                
                
            }else if (!strcmp_case(prog,"DTREK")){

                printf("D*TREK output Log file (%s)\n",infile);
                extract_DTREK_log(infile); 
                        
            }else if(!strcmp_case(prog,"SAINT")){

                printf("SAINT output Log file (%s)\n",infile);
                extract_SAINT_log(infile);
                        
            }else if(!strcmp_case(prog,"X-GEN") || !strcmp_case(prog,"XGEN")||
                     !strcmp_case(prog,"XENGEN")){

                printf("X-GEN output Log file (%s)\n",infile);
                extract_XGEN_log(infile);
                        
            }else if(!strcmp_case(prog,"SCALA")){

                printf("SCALA output Log file (%s)\n",infile);
                extract_SCALA_log(infile);
                
            }else if(!strcmp_case(prog,"AIMLESS") ){

                printf("AIMLESS output Log file (%s)\n",infile);
                extract_AIMLESS_log(infile);
                        
            }else if(!strcmp_case(prog,"3DSCALE")){

                printf("3DSCALE output Log file (%s)\n",infile);
                
                extract_3DSCALE_log(infile);

            }else if(!strcmp_case(prog,"XSCALE") || !strcmp_case(prog,"XDS")){

                printf("XSCALE output Log file (%s)\n",infile);
                extract_XSCALE_log(infile); //one data set for refinement
//                printf("\nNote:The default sigma cutoff for Xscale is -3.0.\n");
//                printf("     If you used other cutoff, please add one more option (e.g. -datacut 0.0)\n\n");
                
            }else if(!strcmp_case(prog,"PROSCALE")){

                printf("PROSCALE output Log file (%s)\n",infile);
                extract_PROSCALE_log(infile);
                
            }else if(!strcmp_case(prog,"XPREP")){

                printf("XPREP output Log file (%s)\n",infile);
                extract_XPREP_log(infile); 

            }else if(!strcmp_case(prog,"marscale")){

                printf("marScale output Log file (%s)\n",infile);
                extract_MARSCALE_log(infile);
                
            }else if(!strcmp_case(prog,"xia") || !strcmp_case(prog,"xia2")){

                printf("XIA2 output Log file (%s)\n",infile);
                extract_XIA2_log(infile); 

            }else if (!strcmp_case(prog,"any")){

                extract_HKL_log(infile);
                if (reflns_shell.nbin>0) {break; strcpy(prog,"HKL");
                }
                extract_XSCALE_log(infile); 
                if (reflns_shell.nbin>0) {break;strcpy(prog,"XSCALE");
                }
                extract_AIMLESS_log(infile);
                if (reflns_shell.nbin>0) {break;strcpy(prog,"AIMLESS");
                }
                extract_SCALA_log(infile);
                if (reflns_shell.nbin>0) break;
                extract_XIA2_log(infile); 
                if (reflns_shell.nbin>0) break;
                extract_MARSCALE_log(infile);
                if (reflns_shell.nbin>0) break;
                extract_DTREK_log(infile); 
                if (reflns_shell.nbin>0) break;
                extract_SAINT_log(infile);
                if (reflns_shell.nbin>0) break;
                extract_XGEN_log(infile);
                if (reflns_shell.nbin>0) break;
                extract_3DSCALE_log(infile);
                if (reflns_shell.nbin>0) break;
                extract_PROSCALE_log(infile);
                if (reflns_shell.nbin>0) break;
                extract_XPREP_log(infile); 
 
            

            }else{
                sprintf(info,"Warning! the program (%s) is not supported!\n",prog);
                pinfo(info, 0);
            }
            
        }else if(!strcmp_case(argv[i],"-IDAT") ){ //follow SF file to calculate I/SigI for HKL
            
            strcpy(infile, argv[i+1]);
            if(!strcmp_case(prog, "HKL")|| !strcmp_case(prog,"SCALEPACK")||
               !strcmp_case(prog, "HKL2000")){
                
                printf("HKL/Scalepack output data file: (%s)\n",infile);
                if(key==0){
                    printf("Input was wrong! You should give the right order\n");
                    printf("pdb_extract -s HKL -ilog  log_file -idat data_file\n");
                    exit(0);
                }
                I_over_sigmaI(infile); //add I/SigmaI and with shell.
            } 
            
        }else if (!strcmp_case(argv[i],"-ICIF") ){
            strcpy(infile, argv[i+1]);
            
            if(!strcmp_case(prog,"SCALA")){
                write_cif(fout, infile, "scala");
            }else
                write_cif(fout, infile, "");
        }
        
    }
    
    if (reflns_shell.nbin<=0) {
        sprintf(info,"Warning! No statistics is extracted from data reduction file! (check program name or file format).\n");
        pinfo(info, 0);
        
    }
}


void extract_scale_phase(FILE *fout, int argc, char **argv, int start, char *prog)
// extract LOG files from many diffraction data set. (option -sp)
{
    char infile[512];
    int i=0, ns=0;
    

    ns=0;
    if(!strcmp_case(argv[start],"-ILOG") ){
        
        for(i=start+1; i<argc; i++){
            strcpy(infile, argv[i]);
            
            if(argv[i][0] =='-') break;
            
            if(!strcmp_case(prog, "HKL")|| !strcmp_case(prog,"SCALEPACK")||
               !strcmp_case(prog, "HKL2000")){
                printf("HKL/Scalepack output Log file for phasing (%s).\n",infile);
                extract_HKL_log(infile);
                
            }else if(!strcmp_case(prog,"DTREK")){
                printf("D*TREK output Log file for phasing (%s).\n",infile);
                extract_DTREK_log(infile);
                
                
            }else if(!strcmp_case(prog,"SCALA")){
                printf("SCALA output Log file for phasing (%s).\n",infile);
                extract_SCALA_log(infile);
                
            }else if(!strcmp_case(prog,"AIMLESS")){
                printf("AIMLESS output Log file for phasing (%s).\n",infile);
                extract_AIMLESS_log(infile);
                
            }else if(!strcmp_case(prog,"3DSCALE")){
                printf("3DSCALE output Log file for phasing (%s).\n",infile);
                extract_3DSCALE_log(infile);

                
            }else if(!strcmp_case(prog,"XSCALE")){
                printf("XSCALE output Log file for phasing (%s).\n",infile);
                extract_XSCALE_log(infile);
                
            }else if(!strcmp_case(prog,"PROSCALE")){
                printf("PROSCALE output Log file for phasing (%s).\n",infile);
                extract_PROSCALE_log(infile);
                
            }
            if(reflns_shell.nbin>0){
                reflns_phs[ns]=reflns;
                reflns_shell_phs[ns]=reflns_shell;
                if(ns++>20) return;
            }
            
        }
    }

    if(ns==0){
        printf("Warning! no information has been extracted from phasing sets!\n");
        return;
    }
    
    for(i=0; i<ns; i++){
        write_diffrn_shell(fout,reflns_shell_phs,i);
    }
    for(i=0; i<ns; i++){
        write_diffrn(fout, reflns_phs, i);
    }
}

void write_diffrn(FILE *fout, REFLNS *log,  int i )
{
    if(i==0 ){
        fprintf(fout, "\nloop_\n");
        fprintf(fout, "_diffrn_reflns.diffrn_id\n");
        fprintf(fout, "_diffrn_reflns.pdbx_d_res_high \n");
        fprintf(fout, "_diffrn_reflns.pdbx_d_res_low \n");
        fprintf(fout, "_diffrn_reflns.pdbx_number_obs \n");
        fprintf(fout, "_diffrn_reflns.pdbx_Rmerge_I_obs \n");
        fprintf(fout, "_diffrn_reflns.pdbx_Rsym_value \n");
        fprintf(fout, "_diffrn_reflns.pdbx_chi_squared \n");
        fprintf(fout, "_diffrn_reflns.pdbx_redundancy \n");
        fprintf(fout, "_diffrn_reflns.pdbx_rejects  \n");
        fprintf(fout, "_diffrn_reflns.pdbx_percent_possible_obs \n");
        fprintf(fout, "_diffrn_reflns.pdbx_observed_criterion \n");
        fprintf(fout, "_diffrn_reflns.number \n");

        fprintf(fout, "_diffrn_reflns.limit_h_max  \n");
        fprintf(fout, "_diffrn_reflns.limit_h_min  \n");
        fprintf(fout, "_diffrn_reflns.limit_k_max  \n");
        fprintf(fout, "_diffrn_reflns.limit_k_min  \n");
        fprintf(fout, "_diffrn_reflns.limit_l_max  \n");
        fprintf(fout, "_diffrn_reflns.limit_l_min  \n");

        
/*  //skip them, not in the dic 
        fprintf(fout, "_diffrn_reflns.Rmerge_F_obs  \n");

        fprintf(fout, "_diffrn_reflns.pdbx_netI_over_av_sigmaI  \n");
        fprintf(fout, "_diffrn_reflns.pdbx_Rrim_I_all \n");
        fprintf(fout, "_diffrn_reflns.pdbx_Rmerge_F_obs \n");
      
        fprintf(fout, "_diffrn_reflns.pdbx_resolution_low_scale \n");
        fprintf(fout, "_diffrn_reflns.pdbx_resolution_high_scale \n");
        
        fprintf(fout, "_diffrn_reflns.pdbx_number_centric \n");
        fprintf(fout, "_diffrn_reflns.pdbx_number_anomalous \n");
        fprintf(fout, "_diffrn_reflns.pdbx_Rmerge_I_anomalous \n");
        fprintf(fout, "_diffrn_reflns.pdbx_netI_over_av_sigmaI_anomalous \n");
        fprintf(fout, "_diffrn_reflns.pdbx_PCV_mean \n");
        fprintf(fout, "_diffrn_reflns.pdbx_netI_over_sigmaI \n");
        fprintf(fout, "_diffrn_reflns.pdbx_average_I_obs \n");
        fprintf(fout, "_diffrn_reflns.pdbx_average_sigmaI_obs \n");

        fprintf(fout, "_diffrn_reflns.R_free_details   \n");
        fprintf(fout, "_diffrn_reflns.details   \n");
        fprintf(fout, "_diffrn_reflns.B_iso_Wilson_estimate \n");
        fprintf(fout, "_diffrn_reflns.Rmerge_F_all  \n");

        
*/        
    }
    fprintf(fout, " %d ",i+1);
    
    (log[i].d_resolution_high>0)?
        fprintf(fout, " %.3f ", log[i].d_resolution_high) : fprintf(fout," . ");
    
    (log[i].d_resolution_low>0) ? 
        fprintf(fout, " %.3f ", log[i].d_resolution_low) : fprintf(fout," . ");
    
    
    (log[i].number_obs>0) ? 
        fprintf(fout, "  %d ", log[i].number_obs) : fprintf(fout," ? ");
    
    (log[i].pdbx_Rmerge_I_obs>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_Rmerge_I_obs) : fprintf(fout," ? ");
    
    (log[i].pdbx_Rsym_value>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_Rsym_value) : fprintf(fout," ? ");
    
    (log[i].pdbx_chi_squared>0) ? 
        fprintf(fout, " %.2f ", log[i].pdbx_chi_squared) : fprintf(fout," ? ");
    
    (log[i].pdbx_redundancy>0) ? 
        fprintf(fout, " %.2f ", log[i].pdbx_redundancy) : fprintf(fout," ? ");
    
    (log[i].pdbx_scaling_rejects>0) ? 
        fprintf(fout, "  %d ", log[i].pdbx_scaling_rejects) : fprintf(fout," ? ");
    
    (log[i].percent_possible_obs>0) ? 
        fprintf(fout, " %.2f ", log[i].percent_possible_obs) : fprintf(fout," ? ");

    (log[i].observed_criterion[0]!='?')? 
        fprintf(fout, " '%s'  ", log[i].observed_criterion) : fprintf(fout," ? ");

    (log[i].pdbx_number_measured_all>0) ? 
        fprintf(fout, " %d ", log[i].pdbx_number_measured_all) : fprintf(fout," ? ");


    (log[i].limit_h_max>INI) ?
        fprintf(fout, " %d ",log[i].limit_h_max) : fprintf(fout," ? ");
    (log[i].limit_h_min>INI) ?
        fprintf(fout, " %d ",log[i].limit_h_min) : fprintf(fout," ? ");
    (log[i].limit_k_max>INI) ?
        fprintf(fout, " %d ",log[i].limit_k_max) : fprintf(fout," ? ");
    (log[i].limit_k_min>INI) ?
        fprintf(fout, " %d ",log[i].limit_k_min) : fprintf(fout," ? ");
    (log[i].limit_l_max>INI) ?
        fprintf(fout, " %d ",log[i].limit_l_max) : fprintf(fout," ? ");
    (log[i].limit_l_min>INI) ?
        fprintf(fout, " %d ",log[i].limit_l_min) : fprintf(fout," ? ");

    
/*

    (log[i].Rmerge_F_obs>0) ? 
        fprintf(fout, " %.3f ", log[i].Rmerge_F_obs) : fprintf(fout," ? ");

    
    (log[i].pdbx_netI_over_av_sigmaI>0) ? 
        fprintf(fout, "  %.2f ", log[i].pdbx_netI_over_av_sigmaI) : fprintf(fout," ? ");
    
    (log[i].pdbx_Rrim_I_all>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_Rrim_I_all) : fprintf(fout," ? ");

    (log[i].pdbx_Rmerge_F_obs>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_Rmerge_F_obs) : fprintf(fout," ? ");

    

    (log[i].pdbx_resolution_low_scale>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_resolution_low_scale) : fprintf(fout," ? ");
    
    (log[i].pdbx_resolution_high_scale>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_resolution_high_scale) : fprintf(fout," ? ");

    
    (log[i].pdbx_number_centric>0) ? 
        fprintf(fout, " %d ", log[i].pdbx_number_centric) : fprintf(fout," ? ");
    
    (log[i].pdbx_number_anomalous>0) ? 
        fprintf(fout, " %d ", log[i].pdbx_number_anomalous) : fprintf(fout," ? ");

    (log[i].pdbx_Rmerge_I_anomalous>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_Rmerge_I_anomalous) : fprintf(fout," ? ");
    
    (log[i].pdbx_netI_over_av_sigmaI_anomalous>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_netI_over_av_sigmaI_anomalous) : fprintf(fout," ? ");

    (log[i].pdbx_PCV_mean>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_PCV_mean) : fprintf(fout," ? ");
    
    (log[i].pdbx_netI_over_sigmaI>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_netI_over_sigmaI) : fprintf(fout," ? ");
    
    (log[i].pdbx_average_I_obs>0) ? 
        fprintf(fout, " %.3f ", log[i].pdbx_average_I_obs) : fprintf(fout," ? ");

    (log[i].pdbx_average_sigmaI_obs>0)? 
        fprintf(fout, " %.3f ", log[i].pdbx_average_sigmaI_obs) : fprintf(fout," ? ");
    
    (log[i].R_free_details[0]!='?')? 
        fprintf(fout, " '%s'  ", log[i].R_free_details) : fprintf(fout," ? ");
    (log[i].details[0]!='?')? 
        fprintf(fout, " '%s'  ", log[i].details) : fprintf(fout," ? ");
    
    (log[i].B_iso_Wilson_estimate>0) ? 
        fprintf(fout, " %.2f ", log[i].B_iso_Wilson_estimate) : fprintf(fout," ? ");
    
    (log[i].Rmerge_F_all>0) ? 
        fprintf(fout, " %.3f ", log[i].Rmerge_F_all) : fprintf(fout," ? ");
        

        
    (log[i].number_all>0) ? 
        fprintf(fout, " %d  ", log[i].number_all) : fprintf(fout," ? ");

    
*/
    fprintf(fout, " \n");
    
}


void write_diffrn_shell(FILE *fout, REFLNS_SHELL *log,  int i)
//Note: value reflns_shell.number_unique_all will be extracted (complimise ADIT)
{
    int m;
    
    if(i==0 && log[i].nbin>0){
        
    fprintf(fout, "\n# statistics from LOG files for phasig\n");
    fprintf(fout, "\nloop_\n");
    
    fprintf(fout, "_pdbx_diffrn_reflns_shell.diffrn_id \n");

    fprintf(fout, "_pdbx_diffrn_reflns_shell.d_res_high\n");
    fprintf(fout, "_pdbx_diffrn_reflns_shell.d_res_low\n");

    fprintf(fout, "_pdbx_diffrn_reflns_shell.number_obs \n");
    fprintf(fout, "_pdbx_diffrn_reflns_shell.rejects\n");
    fprintf(fout, "_pdbx_diffrn_reflns_shell.Rmerge_I_obs\n");
    
    fprintf(fout, "_pdbx_diffrn_reflns_shell.Rsym_value \n");
    fprintf(fout, "_pdbx_diffrn_reflns_shell.chi_squared\n");
    fprintf(fout, "_pdbx_diffrn_reflns_shell.redundancy\n");
    fprintf(fout, "_pdbx_diffrn_reflns_shell.percent_possible_obs\n");

/*    
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.number_measured_obs\n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.number_measured_all\n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.meanI_over_sigI_obs \n");
    

//    fprintf(fout, "_pdbx_diffrn_reflns_shell.Rmeas_mean \n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.meanI_over_sd \n");
    
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.number_centric \n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.number_anomalous \n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.Rmerge_I_anomalous \n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.meanI_over_sigI_anomalous \n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.PCV_mean \n");
    
        
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.number_possible\n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.number_unique_all\n");
   
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.Rmerge_F_all\n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.Rmerge_F_obs\n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.Rmerge_I_all\n");
    
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.meanI_over_sigI_all\n");
//    fprintf(fout, "_pdbx_diffrn_reflns_shell.percent_possible_all\n");
*/
    
    }
    
        
    for(m=0; m<log[i].nbin; m++){
        fprintf(fout," %d ", i+1);
        
        (log[i].d_res_high[m]>0) ?
            fprintf(fout," %.2f ", log[i].d_res_high[m]) : fprintf(fout," ? ");
        (log[i].d_res_low[m]>0) ?
            fprintf(fout," %.2f ", log[i].d_res_low[m]) : fprintf(fout," ? ");
        
        (log[i].number_unique_obs[m]>0) ?
            fprintf(fout," %d ",log[i].number_unique_obs[m]): fprintf(fout," ? ");

        (log[i].pdbx_rejects[m]>0) ?
            fprintf(fout," %d ", log[i].pdbx_rejects[m]): fprintf(fout," ? ");


        (log[i].Rmerge_I_obs[m]>0) ?
            fprintf(fout," %.3f ",log[i].Rmerge_I_obs[m]): fprintf(fout," ? ");

        
        (log[i].pdbx_Rsym_value[m]>0) ?
            fprintf(fout," %.3f \n",log[i].pdbx_Rsym_value[m]):fprintf(fout," ? ");
        (log[i].pdbx_chi_squared[m]>0) ?
            fprintf(fout," %.3f ", log[i].pdbx_chi_squared[m]): fprintf(fout," ? ");
        (log[i].pdbx_redundancy[m]>0) ?
            fprintf(fout," %.2f ", log[i].pdbx_redundancy[m]): fprintf(fout," ? ");
        
        (log[i].percent_possible_obs[m]>0) ?
            fprintf(fout," %.2f ", log[i].percent_possible_obs[m]): fprintf(fout," ? ");

/*

        (log[i].number_measured_obs[m]>0) ?
           fprintf(fout," %d ", log[i].number_measured_obs[m]): fprintf(fout," ? ");
        (log[i].number_measured_all[m]>0) ?
            fprintf(fout," %d ", log[i].number_measured_all[m]): fprintf(fout," ? ");
        (log[i].meanI_over_sigI_obs[m]>0) ?
            fprintf(fout," %.1f ", log[i].meanI_over_sigI_obs[m]): fprintf(fout," ? ");

        
        (log[i].pdbx_Rrim_I_all[m]>0) ?
            fprintf(fout," %.3f ", log[i].pdbx_Rrim_I_all[m]): fprintf(fout," ? ");

        (log[i].pdbx_netI_over_sigmaI_obs[m]>0) ?
            fprintf(fout," %.1f ", log[i].pdbx_netI_over_sigmaI_obs[m]): fprintf(fout," ? ");


        
        (log[i].pdbx_number_centric[m]>0) ?
            fprintf(fout," %d ", log[i].pdbx_number_centric[m]): fprintf(fout," ? ");

        (log[i].pdbx_number_anomalous[m]>0) ?
            fprintf(fout," %d ",log[i].pdbx_number_anomalous[m] ): fprintf(fout," ? ");

        (log[i].pdbx_Rmerge_I_anomalous[m]>0) ?
            fprintf(fout," %.2f ",log[i].pdbx_Rmerge_I_anomalous[m] ): fprintf(fout," ? ");

        (log[i].pdbx_meanI_over_sigI_anomalous[m]>0) ?
            fprintf(fout," %.1f ",log[i].pdbx_meanI_over_sigI_anomalous[m] ): fprintf(fout," ? ");

        (log[i].pdbx_PCV_mean[m]>0) ?
            fprintf(fout," %.3f ",log[i].pdbx_PCV_mean[m] ): fprintf(fout," ? ");

        
        (log[i].number_possible[m]>0) ?
            fprintf(fout," %d ",log[i].number_possible[m]): fprintf(fout," ? ");
        
        (log[i].number_unique_all[m]>0) ?
            fprintf(fout," %d ", log[i].number_unique_all[m]): fprintf(fout," ? ");

        
        (log[i].Rmerge_F_all[m]>0) ?
            fprintf(fout," %.3f ",log[i].Rmerge_F_all[m]): fprintf(fout," ? ");
        
        (log[i].Rmerge_F_obs[m]>0) ?
            fprintf(fout," %.3f ",log[i].Rmerge_F_obs[m]): fprintf(fout," ? ");
        
        (log[i].Rmerge_I_all[m]>0) ?
            fprintf(fout," %.3f ",log[i].Rmerge_I_all[m]): fprintf(fout," ? ");

        (log[i].meanI_over_sigI_all[m]>0) ?
            fprintf(fout," %.1f ", log[i].meanI_over_sigI_all[m]): fprintf(fout," ? ");
        

        (log[i].percent_possible_all[m]>0) ?
            fprintf(fout," %.2f ", log[i].percent_possible_all[m]): fprintf(fout," ? ");
*/

        fprintf(fout," \n");
        
    }
}

//================================================

void add_value2cell(float a, float b,float c,float alpha,float beta,float gamma)
{

    cell.length_a = a;
    cell.length_b = b;
    cell.length_c = c;
    cell.angle_alpha = alpha;
    cell.angle_beta = beta;
    cell.angle_gamma = gamma;
}

void extract_HKL_log( char *inpfile )
// Extracting statistics from LOG files of HKL or SCALEPACK 
{
    char  tmp[512], **line;
    int i, j, ntmp=0,k1,  k=0, n=0,n1, nline, nbin=0;
    float a1, a2, a3, a4, a5,a6,a7, a8,a9;

    reflns_shell.nbin=0;
    line=get_lines_from_file(inpfile, &nline);
    
    for(i=0; i<nline-1; i++){
        if(strstr_space(line[i], "Space group")){
            if(pattern5(line[i],"Space group", "number", "number", "","")){
                strcpy(tmp, get_substr(line[i],"group", "number"));
                if(strlen(tmp)>0) strcpy(symmetry.space_group_name_H_M, tmp);
                strcpy(tmp, "");
                strcpy(tmp, get_substr(line[i],"number", "number"));
                if(strlen(tmp)>0)symmetry.Int_Tables_number = atoi(tmp);
            }
            
        }else if(strstr_space(line[i], "Unit cell volume")){
            if(i>1){
                k=sscanf(line[i-1], "%*s%f%f%f%f%f%f %*s%*s%*s %f",
                         &a1, &a2, &a3, &a4, &a5, &a6, &a7);
                if( k==7 ){
                    add_value2cell(a1,a2,a3,a4,a5,a6);
                    exptl_crystal.pdbx_mosaicity  = a7;
                }
            }
            
        }else if(strstr_space(line[i], "All films")){
            k=sscanf(line[i],"%*s %*s %d %*s %f %f %f", &n1, &a1, &a2,  &a3);
            if(k==4){
                reflns.pdbx_number_measured_all=n1;
                reflns.pdbx_netI_over_sigmaI=a1;
                reflns.pdbx_chi_squared=a2;
            }
  
        }else if(strstr_case(line[i], "Summary of observation redundancies by shells:") &&
                 strstr_case(line[i+1], "No. of reflections") ){
            
            n=0;
            for(j=i+2; j<nline; j++){ //skip up to limit
                if(strstr(line[j], "limit"))  continue;
                
                if(n>=NBIN){
                    n=0;
                    break;
                }
                if(strstr_space(line[j], "All hkl")){
                    reflns.number_obs = get_int_by_length(line[j], 73, 79);
                    i=j;
                    break;
                }
                
                reflns_shell.d_res_low[n] =  get_float_by_length(line[j], 0, 6) ;
                reflns_shell.d_res_high[n] =  get_float_by_length(line[j], 7, 12) ;
                reflns_shell.number_unique_obs[n] = get_int_by_length(line[j], 73, 79);
                n++;
            }
            nbin=n;
            
  
        }else if(strstr_case(line[i], "I/Sigma in resolution shells:") &&
                 strstr_case(line[i+1], "No. of reflections") ){
            
            n=0;
            for(j=i+2; j<nline; j++){ //skip up to limit
                if(strstr(line[j], "limit"))  continue;
                
                if(n>=NBIN){
                    n=0;
                    break;
                }
                if(strstr_space(line[j], "All hkl")){
                    reflns.number_obs = get_int_by_length(line[j], 61, 68);
                    i=j;
                    break;
                }
                
                reflns_shell.d_res_low[n] =  get_float_by_length(line[j], 0, 6) ;
                reflns_shell.d_res_high[n] =  get_float_by_length(line[j], 7, 12) ;
                reflns_shell.number_unique_obs[n] = get_int_by_length(line[j], 61, 68);
                n++;
            }
            nbin=n;

        }else if(strstr_space(line[i], "Shell       Average Redundancy Per Shell")){
            if( strstr_space(line[i+1], "Lower Upper")){
                n=0;
                for(j=i+2; j<nline; j++){ //skip up to limit
                    if(strstr(line[j], "limit"))  continue;
                    if(n>=NBIN){
                        n=0;
                        break;
                    }
                    if(strstr_space(line[j], "All hkl")){
                        reflns.pdbx_redundancy = get_float_by_length(line[j], 12, 29);
                        i=j;
                        break;
                    }
                    k=sscanf(line[j], "%f%f%f",&a1, &a2, &a3);
                    if(k==3){
                        reflns_shell.d_res_low[n] =  a1;
                        reflns_shell.d_res_high[n] =  a2 ;
                        reflns_shell.pdbx_redundancy[n] =  a3 ;
                        
                        n++;
                    }
                }
                nbin=n;
            }
            
        }else if(strstr_space(line[i], "Summary of observation redundancies:")){

            if( strstr_space(line[i+1], "% of reflections with given No")){
                n=0;
                for(j=i+2; j<nline; j++){ 
                    if(strstr(line[j], "limit"))  continue;
                    if(n>=NBIN){
                        n=0;
                        break;
                    }
                    if(strstr_space(line[j], "All hkl")){
                        reflns.percent_possible_obs = get_float_by_length(line[j], 73, 79);
                        i=j;
                        break;
                    }
                    reflns_shell.percent_possible_all[n] = get_float_by_length(line[j], 73, 79);
                    n++;
                }
                nbin=n;
            }
            
        }else if(strstr_space(line[i], "Shell Lower Upper Average")){  
            ntmp=0;
            if(strstr_space(line[i+1], "limit    Angstrom")){
                if (strstr_space(line[i+1], "Rmeas   Rpim  CC1/2")) ntmp=1;
                
                n=0;
                for(j=i+2; j<nline; j++){ 
                    if(strstr(line[j], "limit"))  continue;
                    if(n>=NBIN){
                        n=0;
                        break;
                    }
                    if(strstr_space(line[j], "All reflections")){
                        k=sscanf(line[j] +18, "%f%f %*s %f%f",&a1, &a2, &a3, &a4);
                        if(k==4){
                            reflns.pdbx_average_I_obs = a1;
                            reflns.pdbx_average_sigmaI_obs = a2;
                            reflns.pdbx_chi_squared = a3;
                            reflns.pdbx_Rmerge_I_obs = a4;

                            if (ntmp>0){
                                k1=sscanf(line[j]+63, "%f%f ",&a5, &a6);
                                if(k1==2){
                                    reflns.pdbx_Rrim_I_all=a5 ;
                                    reflns.pdbx_Rpim_I_all=a6 ;
                                }
                            }

                        }
                    
                        i=j;
                        break;
                    }
                    k=sscanf(line[j], "%f%f%f%f  %*s %f%f",&a1, &a2, &a3, &a4, &a5, &a6);
                    if(k==6){
                        reflns_shell.d_res_low[n] =  a1;
                        reflns_shell.d_res_high[n] =  a2 ;
                        reflns_shell.pdbx_average_I_obs[n] =  a3 ;
                        reflns_shell.pdbx_average_sigmaI_obs[n] =  a4 ;
                        reflns_shell.pdbx_chi_squared[n] =  a5 ;
                        reflns_shell.Rmerge_I_obs[n] =  a6 ;

                        if (ntmp>0){
                            k1=sscanf(line[j]+63, "%f%f%f ",&a7, &a8, &a9);
                            if(k1==3){
                                reflns_shell.pdbx_Rrim_I_all[n]=a7 ;
                                reflns_shell.pdbx_Rpim_I_all[n]=a8;
                                reflns_shell.pdbx_CC_half[n]=a9; 
                            }
                        }
                        n++;
                    }  
                }
                nbin=n;
            }
        }
    }
    if(reflns_shell.d_res_low[0]>0){
        reflns.d_resolution_low=reflns_shell.d_res_low[0];
        reflns.pdbx_resolution_low_scale=reflns_shell.d_res_low[0];
    }
    
    if(reflns_shell.d_res_high[nbin-1]>0){
        reflns.d_resolution_high=reflns_shell.d_res_high[nbin-1];
        reflns.pdbx_resolution_high_scale=reflns_shell.d_res_high[nbin-1];
    }
    
    if(nbin >0){
        reflns_shell.nbin=nbin;
    }else{
        reflns_shell.nbin=0;
        printf("Warning! Nothing extracted from (%s)!\n", inpfile);
    }
    if(nline>0)free_memory_2d(line, nline);
    
    
}


void extract_MARSCALE_log( char *inpfile )
// Extracting statistics from LOG files of MARSCALE
{
    char  tmp[256], **line;
    int i,j, k=0,n, n1=0,n2=0,n3=0,n4=0,n5=0, nline;
    float a1, a2, a3, a4, a5,a6;

    reflns_shell.nbin=0;
    line=get_lines_from_file(inpfile, &nline);
    
    for(i=0; i<nline-1; i++){
        if(pattern5(line[i], "Space group:"  , "(#", ")", "Point group:", "" )){
                strcpy(tmp, get_substr(line[i],"group:", "("));
                if(strlen(tmp)>0) strcpy(symmetry.space_group_name_H_M, tmp);
                
                strcpy(tmp, "");
                strcpy(tmp, get_substr(line[i],"(#", ")"));
                if(strlen(tmp)>0)symmetry.Int_Tables_number = atoi(tmp);
            
        }else if(strstr_space(line[i], "Cell constants :")){
            strcpy(tmp, strstr(line[i], "ts :")+4);
            k=sscanf(tmp, "%f%f%f%f%f%f",   &a1, &a2, &a3, &a4, &a5, &a6);
            if( k==6 ){ add_value2cell(a1,a2,a3,a4,a5,a6);}

        }else if(strstr_space(line[i], "resolution shells defined")){
            if(sscanf(line[i],"%d", &n1)==1){
                if(n1>0 && n1<NBIN)reflns_shell.nbin=n1;
            }
        }else if(strstr_space(line[i], "reflexions read from input")){
            if(sscanf(line[i],"%d", &n1)==1){
                if(n1>10)reflns.pdbx_number_measured_all=n1;
            }
            
        }else if(pattern5(line[i],"=> actual highest resolution =","A =>" , "", "","")){
            reflns.d_resolution_high=get_float_after_id(line[i], "tion =");
            
        }else if(pattern5(line[i],"& lowest resolution =","A =>" , "", "","")){
            reflns.d_resolution_low=get_float_after_id(line[i], "tion =");
            reflns_shell.d_res_low[0]= reflns.d_resolution_low;
            
        }else if(strstr_space(line[i], "Multiplicity of accepted measurements for unique reflexions")){
            n=0;
            for(j=i+2; j<nline; j++){ //skip up to limit
                if(strstr(line[j], "limits"))  continue;
                if(n>=NBIN || strstr(line[j], "----------- ----- ----- -----")){
                    k=sscanf(line[j+1]+67,"%d%f", &n1, &a1);
                    if(k==2){
                        reflns.number_obs = n1;
                        reflns.percent_possible_obs = a1;
                    }
                    i=j+1;
                    break;
                }else if(pattern5(line[j], " - ", ".", ".","", "")){
                    
                    k=sscanf(line[j],"%s %*s %f %*s%*s%*s%*s%*s%*s%*s%*s%*s %d%f",tmp, &a1,  &n1, &a2);
                    if(k==4){
                        if(n!=0) reflns_shell.d_res_low[n] = atof(tmp);
                        reflns_shell.d_res_high[n] =  a1 ;
                        reflns_shell.number_unique_obs[n] = n1;
                        reflns_shell.percent_possible_all[n] = a2 ;
                        n++;
                    }
                }
            }
            n2=n;
            
        }else if(strstr_space(line[i], "Same multiplicities as relative percentages")){
            n=0;
            for(j=i+2; j<nline; j++){
                if(n>=NBIN || strstr(line[j], "-----------  ----- ----- ----- ----- ----- -----")){
                    if(pattern5(line[j+1]+73, "<", ".", ">", "","")){
                        get_data_set_id(line[j+1]+73, "<", ">", tmp);
                        reflns.pdbx_redundancy =atof(tmp);
                        i=j+1;
                        break;
                    }
                }else if(pattern5(line[j], " - ", ".", ".","<", ">")){
                    get_data_set_id(line[j], "<", ">", tmp);
                    k=sscanf(tmp,"%f", &a1);
                    if(k==1){
                        reflns_shell.pdbx_redundancy[n] = a1 ;
                        n++;
                    }
                }
            }
            n3=n;
        }else if(strstr_space(line[i], "Table of <<I>/sigma(<I>)> for mean values on output file")){
            n=0;
            for(j=i+4; j<nline; j++){ 
                if(n>=NBIN || strstr(line[j], "-------------  -------------")){
                    k=sscanf(line[j+1]+21,"%f",  &a1);
                    if(k==1){
                        reflns.pdbx_netI_over_sigmaI = a1;
                    }
                    i=j+1;
                    break;
                }else if(pattern5(line[j], " - ",".", ".", ".",".")){
                    k=sscanf(line[j]+21,"%f", &a1);
                    if(k==1){
                        reflns_shell.pdbx_netI_over_sigmaI_obs[n] =  a1;
                        n++;
                    }
                }
            }
            n4=n;
            
            
        }else if(strstr_case(line[i], "Resolution   unique     PCV    R_rim   R_pim   Rw_rim  Rw_pim")||
                 strstr_case(line[i], "Resolution   /+//-/     PCV    R_rim   R_pim   Rw_rim  Rw_pim")){
            n=0;
            for(j=i+2; j<nline; j++){
                if(n>=NBIN || strstr(line[j], "---------  ------- ------- -------  -------")){
                    if(pattern5(line[j+1], "total :", "%", "%", "%","%")){
                        strcpy(tmp, line[j+1]+13);
                        if(strchr(tmp, '%')){
                            replace_char(tmp, '%', ' ');
                        }
                        
                        k=sscanf(tmp,"%d %f%f%f%*s%*s%*s%f",&n1,&a1, &a2, &a3, &a4);
                        if(k==5){
                            reflns.number_obs = n1;
                            reflns.pdbx_PCV_mean= a1*0.01;
                            reflns.pdbx_Rrim_I_all= a2*0.01;
                            reflns.pdbx_Rpim_I_all= a3*0.01;
                            reflns.pdbx_Rmerge_I_obs= a4*0.01;
                        }
                        i=j+1;
                        break;
                    }
                }else if(pattern5(line[j], " - ", ".", ".","", "")){
                    k=sscanf(line[j]+13,"%d %f%f%f%*s%*s%*s%f",&n1,&a1, &a2, &a3, &a4);
                    if(k==5){
                        reflns_shell.number_unique_obs[n] = n1;
                        reflns_shell.pdbx_PCV_mean[n] =a1*0.01;
                        reflns_shell.pdbx_Rrim_I_all[n] =a2*0.01;
                        reflns_shell.pdbx_Rpim_I_all[n] =a3*0.01;
                        reflns_shell.Rmerge_I_obs[n] =a4*0.01;
                        n++;
                    }
                }
            }
            n5=n;
        }
    }
    
    if(n2==n3 && n3==n4 && n4==n5 && n5>0){
        reflns_shell.nbin=n5;
    }else{
        reflns_shell.nbin=0;
        printf("Warning! Nothing extracted from (%s)!\n", inpfile);
    }
    if(nline>0)free_memory_2d(line, nline);
    
    
}


void extract_DTREK_log(char *inpfile )
// Extracting statistics from LOG files of D*trek 
{
    char tmp[512], **lines=NULL;
    long i=0,m1=0, j, m2=0, k=0,   n1, n2,n3;
    int nline;
    float a1, a2, a3, a4, a5, a6;
    
    lines=get_lines_from_file(inpfile, &nline );
    reflns_shell.nbin=0;
    for(i=1; i<nline-3; i++){
        if (strstr_space(lines[i], "Unit cell lengths:")){
            strcpy(tmp, strstr_case(lines[i], "lengths:")+8);
            if(sscanf(tmp, "%f%f%f", &a1, &a2, &a3)==3){
                cell.length_a = a1;
                cell.length_b = a2;
                cell.length_c = a3;
            }
            
        }else if(strstr_space(lines[i], "Unit cell  angles:")){
            strcpy(tmp, strstr_case(lines[i], "angles:")+7);
            if(sscanf(tmp, "%f%f%f", &a1, &a2, &a3)==3){
                cell.angle_alpha = a1;
                cell.angle_beta = a2;
                cell.angle_gamma = a3;
            }
            
        }else if(strstr_space(lines[i], "Mosaicity:")){
            strcpy(tmp, strstr_case(lines[i], "Mosaicity:")+10);
            if(sscanf(tmp, "%f", &a1)==1){
                exptl_crystal.pdbx_mosaicity= a1;
            }
            
        }else if(strstr_space(lines[i], "Spacegroup number:")){
            strcpy(tmp, strstr_case(lines[i], "number:")+7);
            if(sscanf(tmp, "%ld", &n1)==1){
                symmetry.Int_Tables_number = n1;
            }
            if(i<nline-2 && strstr_space(lines[i+1], " name:")){
                get_data_set_id(lines[i+1], "name:", "\n", symmetry.space_group_name_H_M);
            }
            
        }else if(strstr_space(lines[i], "All batches") && !strstr(lines[i],"---") ){
            k=sscanf(lines[i]+14, "%*s %ld %ld %*s %*s %f %f  %*s %f",
                     &n1, &n2, &a1, &a2,  &a3);
            if(k==5){
                reflns.pdbx_number_measured_all=n1;
                reflns.pdbx_scaling_rejects=n2;
                reflns.pdbx_netI_over_sigmaI=a1;
                reflns.pdbx_chi_squared=a2;
                reflns.pdbx_Rrim_I_all=a3;
            }
            
            
        }else if(strstr_space(lines[i], "range      unique     obs   rejs   mults  single unique   mult  shell  cumul") ){
// Completeness vs Resolution
            m1=0;

            for(j=i+2; j<nline-3; j++){
                k=sscanf(lines[j], "%f%*s%f %*s %ld %ld %*s%*s %ld %f%f",
                         &a1,&a2,&n1,&n2,&n3, &a3, &a4);
                
                if(k==7){
                    reflns_shell.d_res_low[m1]=a1;
                    reflns_shell.d_res_high[m1]=a2;
                    reflns_shell.number_measured_all[m1]=n1;
                    reflns_shell.pdbx_rejects[m1]=n2;
                    reflns_shell.number_unique_obs[m1]=n3;
                    reflns_shell.pdbx_redundancy[m1]=a3;
                    reflns_shell.percent_possible_all[m1]=a4;
                    m1++;
                }
                
                if(strstr(lines[j], "--------------") && m1>0){
                    k=sscanf(lines[j+1], "%f%*s%f %*s %ld %ld %*s%*s %ld %f%f",
                         &a1,&a2,&n1,&n2,&n3, &a3, &a4);
                    if(k==7){
                        reflns.d_resolution_low = a1;
                        reflns.d_resolution_high = a2;
                        reflns.pdbx_number_measured_all = n1;
                        reflns.pdbx_scaling_rejects = n2;
                        reflns.number_obs = n3;
                        reflns.pdbx_redundancy = a3;
                        reflns.percent_possible_obs = a4;
                    }
                    i=j;
                    break;
                }
            }
            
        }else if(strstr_space(lines[i], "range      counts     obs   rejs   ovlps   mults <sig>>") ){
//Rmerge vs Resolution
//d*TREK version 7.0SSI -- Feb 21 2001            

            m2=0;

            for(j=i+2; j<nline-3; j++){
                if(sscanf(lines[j], "%f%*s%f %*s%*s%*s%*s%*s %f%f%f",
                          &a1,&a2, &a3, &a4, &a5)==5){
                    
                    reflns_shell.d_res_low[m2]=a1;
                    reflns_shell.d_res_high[m2]=a2;
                    
                    reflns_shell.meanI_over_sigI_obs[m2]=a3;
                    reflns_shell.pdbx_chi_squared[m2]=a4;
                    reflns_shell.Rmerge_I_obs[m2]=a5;

                    m2++;
                }else if(strstr(lines[j], "--------------") && m2>0){
                    if(sscanf(lines[j+1], "%f%*s%f %*s%*s%*s%*s%*s %f%f%f",
                          &a1,&a2, &a3, &a4, &a5)==5){
                        reflns.d_resolution_low=a1;
                        reflns.d_resolution_high=a2;
//                        reflns.pdbx_netI_over_av_sigmaI=a3;
                        reflns.pdbx_netI_over_sigmaI=a3;
                        reflns.pdbx_chi_squared=a4;
                        reflns.pdbx_Rmerge_I_obs=a5;
                        
                  
                    }
                    i=j;
                    break;
                }
            }
        }else if(strstr_space(lines[i], "range      counts   rejs   mults   unavg    avg  ChiSq  Eadd*  shell  cumul") ){
//Rmerge vs Resolution
//  d*TREK version 9.1LDz -- Nov 12 2003;  9.1L -- Jan  3 2004          
            m2=0;

            for(j=i+2; j<nline-3; j++){
                if(sscanf(lines[j], "%f %*s %f %*s%*s%*s%*s %f %f %*s %f",
                          &a1,&a2, &a3, &a4, &a5)==5){
                    
                    reflns_shell.d_res_low[m2]=a1;
                    reflns_shell.d_res_high[m2]=a2;
                    reflns_shell.meanI_over_sigI_obs[m2]=a3;
                    reflns_shell.pdbx_chi_squared[m2]=a4;
                    reflns_shell.Rmerge_I_obs[m2]=a5;
                    m2++;
                }else if(strstr(lines[j], "--------------") && m2>0){
                    if(sscanf(lines[j+1], "%f %*s %f %*s%*s%*s%*s %f %f %*s %f",
                          &a1,&a2, &a3, &a4, &a5)==5){
                       
                        reflns.d_resolution_low=a1;
                        reflns.d_resolution_high=a2;
                        reflns.pdbx_netI_over_sigmaI=a3;
                        reflns.pdbx_chi_squared=a4;
                        reflns.pdbx_Rmerge_I_obs=a5;
                    }
                    i=j;
                    break;
                }
            }
                
        }else if(strstr_space(lines[i], "range    unavg   avg ChiSqA  shell  cumul ChiSq  shell  cumul  shell  cumul") ){
//Merging R factors vs Resolution  
//  d*TREK version 9.9.9.6L -- Jan  3 2013         
            m2=0;
            for(j=i+2; j<nline-3; j++){
                if(sscanf(lines[j], "%f %*s %f %*s %f %*s %*s %*s %f %f %*s %f %*s", &a1,&a2, &a3, &a4, &a5, &a6)==6){

                    reflns_shell.d_res_low[m2]=a1;
                    reflns_shell.d_res_high[m2]=a2;
                    reflns_shell.meanI_over_sigI_obs[m2]=a3;
                    reflns_shell.pdbx_chi_squared[m2]=a4;
                    reflns_shell.Rmerge_I_obs[m2]=a5;
                    reflns_shell.pdbx_Rrim_I_all[m2]=a6;
                    m2++;
                }else if(strstr(lines[j], "--------------") && m2>0){
                    if(sscanf(lines[j+1], "%f %*s %f %*s %f %*s %*s %*s %f %f %*s %f %*s", &a1,&a2, &a3, &a4, &a5, &a6)==6){

                        reflns.d_resolution_low=a1;
                        reflns.d_resolution_high=a2;
                        reflns.pdbx_netI_over_sigmaI=a3;
                        reflns.pdbx_chi_squared=a4;
                        reflns.pdbx_Rmerge_I_obs=a5;
                        reflns.pdbx_Rrim_I_all=a6;
                    }
                    i=j;
                    break;
                }
            }


        }else if(strstr_space(lines[i], "Summary of data collection statistics") ){
            
           for(j=i+2; j<nline-3; j++){
                if(strstr_space(lines[j], "Spacegroup")){
                   get_data_set_id(lines[j], "Spacegroup", "\n",
                                   symmetry.space_group_name_H_M);
                   
                }else if(strstr_space(lines[j], "Resolution range")){
                   if(sscanf(lines[j]+30, "%f%*s%f", &a1, &a2) ==2){
                        reflns.d_resolution_low=a1;
                        reflns.d_resolution_high=a2;
                   }
                   
                }else if(strstr_space(lines[j], "Number of unique reflections")){         
                   strcpy(lines[j], strstr_case(lines[j], "reflections")+11); 
                   if(sscanf(lines[j]+30, "%ld", &n1) ==1) reflns.number_obs=n1;
                   
                }else if(strstr_space(lines[j], "Average redundancy")){
                    if(sscanf(lines[j]+30, "%f", &a1) ==1) reflns.pdbx_redundancy = a1;
                    
                }else if(strstr_space(lines[j], "% completeness")){   
                    if(sscanf(lines[j]+30, "%f", &a1) ==1)  reflns.percent_possible_obs = a1;
                    
                }else if(strstr_space(lines[j], "Rmerge ")){
                    if(sscanf(lines[j]+30, "%f", &a1) ==1) reflns.pdbx_Rmerge_I_obs = a1;
                    
                }else if(strstr_space(lines[j], "Reduced ChiSquared ")){
                    if( sscanf(lines[j]+30, "%f", &a1) ==1) reflns.pdbx_chi_squared = a1;
                    
                }else if(strstr_space(lines[j], "Output <I/sigI>")){
//                    if(sscanf(lines[j]+30, "%f", &a1) ==1) reflns.pdbx_netI_over_av_sigmaI = a1;
                    if(sscanf(lines[j]+30, "%f", &a1) ==1) reflns.pdbx_netI_over_sigmaI = a1;
                    
                }else if(strstr(lines[j], "--------------")){
                    i=j;
                    break;
                }
            }
           
        }
                
    }
    reflns.pdbx_resolution_high_scale=reflns.d_resolution_high;
    reflns.pdbx_resolution_low_scale=reflns.d_resolution_low;
    
    if(m1!=0 && m2!=0 ){
        reflns_shell.nbin = m1;
    }else{
        reflns_shell.nbin=0;
        printf("Warning! Nothing extracted from (%s)!\n", inpfile);
    }
        
    if(lines) free_memory_2d(lines, nline);
    
}


void extract_XGEN_log(char *inpfile )
// Extracting statistics from LOG files of X-GEN
{
    char  **line, tmp[256];
    int nline, i,j, n,k, n1,n2, ns1=0, ns2=0;
    float dat[6];

    line=get_lines_from_file(inpfile, &nline);
    for(i=0; i<nline-1; i++){

        if(pattern5(line[i], "Overall mosaicity =", "degrees",  "", "", "" )){
            get_data_set_id(line[i],"=", "degrees", tmp);
            exptl_crystal.pdbx_mosaicity= atof(tmp);
            
        }else if(pattern5(line[i], "Crystal Spacegroup:",",", "Tables No.", ",", "" )){
            get_data_set_id(line[i],"roup:", ",", tmp);
            if(strlen(tmp)>0) strcpy(symmetry.space_group_name_H_M, tmp);
            get_data_set_id(line[i],"No.", ",", tmp);
            if(strlen(tmp)>0)symmetry.Int_Tables_number = atoi(tmp);
            
        }else if(pattern5(line[i], "Real-Space Parameters:", "a", "alpha", "gamma", "")){
            k=sscanf(line[i+1], "%f%f%f%f%f%f",
                     &dat[0], &dat[1], &dat[2], &dat[3], &dat[4], &dat[5]);
            if( k==6 ){ put_data(dat,"cell"); }
            
        }else if(strstr_space(line[i], "tions, Not Merging Bijvoets   =")){
            reflns.number_obs=get_int_after_id(line[i], "=");
            
        }else if(strstr_space(line[i], "reflections have had ALL observations marked for deletion")){
            if(sscanf(line[i],"%d", &n1)==1){
                reflns.pdbx_scaling_rejects=n1;
            }
            
        }else if(strstr_space(line[i],"Data Run   Number of Observations")){
            if(i<nline-4 && strstr_space(line[i+4],"Totals:")){
                k=sscanf(line[i+4],"%*s %*s %d %*s%*s%*s%*s%*s %f%f%f%f %*s%*s %f",
                         &n1,&dat[0],&dat[1],&dat[2], &dat[3], &dat[4]);
                if(k==6){
                    reflns.pdbx_number_measured_all=n1;
                    reflns.B_iso_Wilson_estimate=dat[0];
                    reflns.pdbx_average_I_obs=dat[1];
                    reflns.pdbx_netI_over_sigmaI=dat[2];
                    reflns.pdbx_chi_squared=dat[3]*0.01;
                    reflns.pdbx_Rmerge_I_obs=dat[4]*0.01;
                }
                
            }
        }else if(strstr_space(line[i],"Limit,A  Resol Possible Collected  Missing Complete ")){
            n=0;
            for(j=i+1; j<nline; j++){
                if(strstr_space(line[j],"Totals:")){
                    if(n>0)reflns.d_resolution_high=reflns_shell.d_res_high[n-1];
                    k=sscanf(line[j],"%*s%*s%*s %d %*s %f %*s%*s%*s%*s%*s%*s%*s%*s %d%f",
                             &n1, &dat[0], &n2, &dat[1]);
                    if(k==4){
                        reflns.number_obs=n1;
                        if(dat[0]<1.001)dat[0]=100*dat[0];
                        reflns.percent_possible_obs=dat[0];
                        reflns.pdbx_number_measured_all=n2;
                        reflns.pdbx_redundancy=dat[1];
                    }
                    i=j;
                    break;
                }
                k=sscanf(line[j],"%f%*s%*s %d %*s %f %*s%*s%*s%*s%*s%*s%*s%*s %d%f",
                         &dat[0], &n1, &dat[1], &n2, &dat[2]);
                if(k==5){
                    reflns_shell.d_res_high[n] = dat[0];
                    if(n==0) reflns_shell.d_res_low[0]=-99999;
                    if(n>0) reflns_shell.d_res_low[n] = reflns_shell.d_res_high[n-1];
                    reflns_shell.number_unique_obs[n] =n1;
                    reflns_shell.number_measured_all[n] =n2 ;
                    
                    if(dat[1]<1.001)dat[1]=100*dat[1];
                    reflns_shell.percent_possible_all[n] = dat[1];
                    reflns_shell.pdbx_redundancy[n] = dat[2];

                    if(n>NBIN-2){
                        printf("Error! Too many shells\n");
                        break;
                    }
                    n++;
                }
            }
            ns1= n;
            
        }else if(strstr_space(line[i],"Summary of Reflection Intensities and R-factors by Shells:")){
            n=0;
            for(j=i+5; j<nline; j++){
                if(strstr_space(line[j],"Totals:")){
                    k=sscanf(line[j],"%*s %f %f %*s%*s%*s%*s%*s%*s%*s%*s %d%d%f %*s%*s %f",
                             &dat[0], &dat[1], &n1, &n2 ,  &dat[2], &dat[3]);
                    if(k==6){
                        reflns.pdbx_average_I_obs=dat[0];
                        reflns.pdbx_netI_over_sigmaI=dat[1];
                        reflns.number_obs=n1;
                        reflns.pdbx_number_measured_all=n2;
                        reflns.pdbx_chi_squared=dat[2]*0.01;
                        reflns.pdbx_Rmerge_I_obs=dat[3]*0.01;
                    }
                    i=j;
                    break;
                    
                }
                k=sscanf(line[j],"%*s %f %f %*s%*s%*s%*s%*s%*s%*s%*s %d%d%f %*s%*s %f",
                         &dat[0], &dat[1], &n1, &n2,   &dat[2], &dat[3]);
                
                if(k==6){
                    reflns_shell.pdbx_netI_over_sigmaI_obs[n] = dat[1];
                    reflns_shell.number_unique_obs[n] = n1;
                    reflns_shell.number_measured_all[n] = n2;
                    reflns_shell.pdbx_chi_squared[n] = dat[2]*0.01;
                    reflns_shell.Rmerge_I_all[n] = dat[3]*0.01;

                    if(n>NBIN-2){
                        printf("Error! Too many shells\n");
                        break;
                    }
                        
                    n++;
                }
            }
            ns2= n;
        }
    }
    if(ns1==ns2 && ns1>0 ){
        reflns_shell.nbin =ns1;
    }else{
        reflns_shell.nbin=0;
        printf("Warning! Nothing extracted from (%s)!\n", inpfile);
    }

    if(nline) free_memory_2d(line,nline);
    
    
}

void extract_SAINT_log(char *inpfile )
// Extracting statistics from LOG files of SAINT
{
    char str[512], tmp[512];
    long k=0, n=0, i1,i2;
    float a1, a2, a3, a4, a5;
    FILE *fp;
    
    if((fp=fopen(inpfile, "r"))==NULL)
       open_file_error(inpfile, "extract_log_SAINT");
    
    while (fgets(str, sizeof str, fp) != NULL) {
        k=0;
        if (strstr_space(str, "Overall    #  Pairs   Uniq   Merg %<2s   <I> <#Sig>   <Bg>  Rsym ")){
            while (fgets(str, sizeof str, fp) != NULL){
                rid_of_front_space(str);
                if(strlen(str)<20) continue;  
                k=sscanf(str,"%*s %ld %*s %ld  %*s%*s  %f%f %*s %f",
                         &i1, &i2, &a1, &a2, &a3);
                if( k==5 ){
                    reflns.pdbx_number_measured_all = i1;
                    reflns.number_obs = i2;
                    if(a1>0 && a2>0) reflns.pdbx_netI_over_av_sigmaI = a1/a2;
                    reflns.pdbx_Rsym_value=a3;
               
                }else{
                    break;
                }
            }
            
        }
        if(k==5)break;
    }
    
    rewind(fp);
    while (fgets(str, sizeof str, fp) != NULL) {
        if (strstr_space(str, "Angstrms   #Obs Theory %Compl Redund   Rsym Pairs %Pairs")){
            n=0;
            while (fgets(str, sizeof str, fp) != NULL){
                rid_of_front_space(str);
                if(n>=NBIN){
                    n=0;
                    break;
                }
                if(strlen(str)<20) continue;
                
                k=sscanf(str,"%s %f%ld%*s %f%f%f%*s%*s%f",
                         tmp, &a1, &i1, &a2, &a3, &a4, &a5);
                
                if( k==7 && !strcmp_case(tmp,"to")){
                    reflns.d_resolution_high=a1;
                    reflns_shell.d_res_high[n]=a1;
                    
                    if(n==0) reflns_shell.d_res_low[0]=-99999;
                    if(n>0) reflns_shell.d_res_low[n] = reflns_shell.d_res_high[n-1];
                    reflns.percent_possible_obs=a2;
                    reflns.pdbx_redundancy=a3;
                    reflns.pdbx_Rsym_value=a4;
                    reflns_shell.pdbx_Rsym_value[n]=a5;
                    n++;
                }else {
                    break;
                }
            }
        }


        
    }
    reflns.pdbx_resolution_high_scale=reflns.d_resolution_high;
    reflns.pdbx_resolution_low_scale=reflns.d_resolution_low;
    
    if(n>0 ){
        reflns_shell.nbin=n;
    }else{
        reflns_shell.nbin=0;
        printf("Warning! Nothing extracted from (%s)!\n", inpfile);
    }
    
}


void extract_SCALA_log(char *inpfile )
/* Extracting statistics from LOG files of SCALA
   Things extracted are :I/sigmaI, completeness, Nrefl, redundancy,Rsym,
   any with resolution shells.
 */  
{
    char  tmp[512], **line;
    char sa0[40], sa1[40], sa2[40], sn1[40], sa3[40], sa4[40], sa5[40], sa6[40];
    int i,j=0, k=0, n=0, n1=0, n2=0,n3=0,  nset1=0, nset2=0, nline;
    float a0,a1, a2, a3, a4, a5, a6;

    line=get_lines_from_file(inpfile, &nline);
    

    for(i=0; i<nline-1; i++){
        if (strstr_space(line[i], " * Cell Dimensions :")){
            k=sscanf(line[i+1],"%f%f%f%f%f%f", &a1, &a2, &a3, &a4, &a5, &a6);
            
            if( k==6 ){ add_value2cell(a1,a2,a3,a4,a5,a6);}
            
        }else if(strstr_space(line[i], "*  Resolution Range :")){
            if(pattern5(line[i+1]," ( ", " - ", "A", ")", "")){
                k=sscanf(line[i+1],"%*s %*s %*s %f %*s %f", &a1, &a2);

                if( k==2 ){
                    reflns.d_resolution_low = a1;
                    reflns.d_resolution_high = a2;
                }
            }
            
        }else if(pattern5(line[i],"Space group =", "'", "'", "(number", ")") ){
            strcpy(tmp, get_substr(line[i],"Space group =", "(number"));
            char2space(tmp, '\'');
            if(strlen(tmp)>0) strcpy(symmetry.space_group_name_H_M, tmp);
            strcpy(tmp, "");
            strcpy(tmp, get_substr(line[i],"number", ")"));
            if(strlen(tmp)>0)symmetry.Int_Tables_number = atoi(tmp);
            
        }else if(strstr_space(line[i], " N 1/d^2 Dmin(A) Rfac  Rfull   Rcum  Ranom  Nanom    Av_I   SIGMA I/sigma") ||
// version before CCP4 5.0
                 strstr_space(line[i], " N 1/d^2 Dmin(A) Rmrg  Rfull   Rcum  Ranom  Nanom    Av_I  SIGMA I/sigma") ){
// version after CCP4 5.0
            n=0;
            for(j=i+1; j<nline-1; j++){
                rid_of_front_end_space(line[j]);
                if(n>=NBIN){
                    n=0;
                    break;
                }
                
                if(!strncmp(line[j], "Overall:", 8) && strstr_space(line[j+1],"SIGMA I/sigma")){
                    
                    k=sscanf(line[j]+9,"%s %*s %*s %s %s %s %s %s %*s %s ",
                             sa1, sa2, sn1, sa3, sa4, sa5, sa6);

                    if(k==7){
                        reflns.pdbx_Rmerge_I_obs = atof(sa1) ;
                        if(isdigit(sa2[0]))reflns.pdbx_Rmerge_I_anomalous = atof(sa2) ;
                        reflns.pdbx_number_anomalous = atoi(sn1) ;
                        reflns.pdbx_average_I_obs = atof(sa3) ;
                        reflns.pdbx_average_sigmaI_obs = atof(sa4) ;
                        reflns.pdbx_netI_over_av_sigmaI= atof(sa5) ;
                        reflns.pdbx_netI_over_sigmaI =  atof(sa6) ;
                    }
                    i=j+1;
                    break;
                }
                if(strlen(line[j])<20  )continue;

                
                k=sscanf(line[j],"%*s %*s %s %s %*s %*s %s %s %s %s %s %*s %s ",
                        sa0, sa1, sa2, sn1, sa3, sa4, sa5, sa6);
                
                if(k==8){
                    if(n==0)reflns_shell.d_res_low[0]=reflns.d_resolution_low;
                    if(n>0) reflns_shell.d_res_low[n] = reflns_shell.d_res_high[n-1];
                    
                    reflns_shell.d_res_high[n] = atof(sa0) ;
                    if(atof(sa0)<=0) continue;
                    
                    reflns_shell.Rmerge_I_obs[n] = atof(sa1) ;
                    if(isdigit(sa2[0]))reflns_shell.pdbx_Rmerge_I_anomalous[n] = atof(sa2) ;
                    reflns_shell.pdbx_number_anomalous[n] = atoi(sn1) ;
                    reflns_shell.pdbx_average_I_obs[n] = atof(sa3) ;
                    reflns_shell.pdbx_average_sigmaI_obs[n] = atof(sa4) ;
                    reflns_shell.meanI_over_sigI_obs[n]= atof(sa5) ;
                    reflns_shell.pdbx_netI_over_sigmaI_obs[n] =  atof(sa6);

                    n++;
                }
            }
            nset1=n;
            
        }else if(strstr_space(line[i],"N 1/resol^2 Dmin    Nmeas    Nref  Ncent   %poss Cm%poss ") &&
                 strstr_space(line[i],"Mlplcty AnomCmpl AnomFrc  Rmeas  Rmeas0  (Rsym)    PCV     PCV0") ){
// version CCP4.2.0 (before 5.0)                 
            n=0;
            for(j=i; j<nline-1; j++){
                if(strlen(line[j])<20  )continue;
                rid_of_front_end_space(line[j]);
                if(n>=NBIN){
                    n=0;
                    break;
                }
               
                if(!strncmp(line[j], "Overall", 7) && strstr_space(line[j+1],"Nmeas    Nref  Ncent")){
                    k=sscanf(line[j]+8,"%d%d%d %f %*s %f %*s%*s%*s  %f %f %*s %f ",
                              &n1, &n2, &n3, &a1, &a2, &a3, &a4, &a5);
                    
                    if(k==8){
                        reflns.pdbx_number_measured_all=n1;
                        reflns.number_obs=n2;
                        reflns.pdbx_number_centric=n3;
                        reflns.percent_possible_obs=a1;
                        reflns.pdbx_redundancy=a2;
                        reflns.pdbx_Rrim_I_all=a3;
                        reflns.pdbx_Rsym_value=a4;
                        reflns.pdbx_PCV_mean=a5;
                    }
                    i=j+1;
                    break;
                }
                
                k=sscanf(line[j],"%*s %*s %f %d%d%d %f %*s %f %*s%*s%*s %f %f %*s %f",
                         &a0,&n1, &n2, &n3,&a1, &a2, &a3, &a4, &a5);

                if(k==9){
                    if(n==0) reflns_shell.d_res_low[0] = reflns.d_resolution_low;
                    if(n>0) reflns_shell.d_res_low[n] = reflns_shell.d_res_high[n-1];
                
                    reflns_shell.d_res_high[n] = a0 ;
                    reflns_shell.number_measured_all[n] = n1 ;
                    reflns_shell.number_unique_obs[n] = n2;
                    reflns_shell.pdbx_number_centric[n] = n3;
                    reflns_shell.percent_possible_all[n] = a1;
                    reflns_shell.pdbx_redundancy[n] =a2 ;
                    reflns_shell.pdbx_Rrim_I_all[n] = a3;
                    reflns_shell.pdbx_Rsym_value[n] = a4;
                    reflns_shell.pdbx_PCV_mean[n] = a5;

                    n++;
                }
            }
            nset2=n;  
        }else if(strstr_space(line[i],"N 1/resol^2 Dmin    Nmeas    Nref  Ncent  %poss C%poss") &&
                 strstr_space(line[i],"Mlplct AnoCmpl AnoFrc  AnoMlt Rmeas  Rmeas0  (Rsym)    PCV     PCV0") ){
// version CCP4 5.0                
            n=0;
            for(j=i; j<nline-1; j++){
                if(strlen(line[j])<20  )continue;
                rid_of_front_end_space(line[j]);
                if(n>=NBIN){
                    n=0;
                    break;
                }
               
                if(!strncmp(line[j], "Overall", 7) && strstr_space(line[j+1],"Nmeas    Nref  Ncent")){
                    k=sscanf(line[j]+8,"%d%d%d %f %*s %f %*s%*s%*s%*s  %f %f %*s %f ",
                              &n1, &n2, &n3, &a1, &a2, &a3, &a4, &a5);
                    
                    if(k==8){
                        reflns.pdbx_number_measured_all=n1;
                        reflns.number_obs=n2;
                        reflns.pdbx_number_centric=n3;
                        reflns.percent_possible_obs=a1;
                        reflns.pdbx_redundancy=a2;
                        reflns.pdbx_Rrim_I_all=a3;
                        reflns.pdbx_Rsym_value=a4;
                        reflns.pdbx_PCV_mean=a5;
                    }
                    i=j+1;
                    break;
                }
                
                k=sscanf(line[j],"%*s %*s %f %d%d%d %f %*s %f %*s%*s%*s%*s %f %f %*s %f",
                         &a0,&n1, &n2, &n3,&a1, &a2, &a3, &a4, &a5);

                if(k==9){
                    if(n==0) reflns_shell.d_res_low[0] = reflns.d_resolution_low;
                    if(n>0) reflns_shell.d_res_low[n] = reflns_shell.d_res_high[n-1];
                
                    reflns_shell.d_res_high[n] = a0 ;
                    reflns_shell.number_measured_all[n] = n1 ;
                    reflns_shell.number_unique_obs[n] = n2;
                    reflns_shell.pdbx_number_centric[n] = n3;
                    reflns_shell.percent_possible_all[n] = a1;
                    reflns_shell.pdbx_redundancy[n] =a2 ;
                    reflns_shell.pdbx_Rrim_I_all[n] = a3;
                    reflns_shell.pdbx_Rsym_value[n] = a4;
                    reflns_shell.pdbx_PCV_mean[n] = a5;

                    n++;
                }
            }
            nset2=n;  
        }else if(strstr_space(line[i],"N 1/resol^2 Dmin    Nmeas    Nref  Ncent  %poss C%poss") &&
// version after CCP4 5.0
                 strstr_space(line[i],"Mlplct AnoCmp AnoFrc AnoMlt Rmeas Rmeas0  (Rsym)  Rpim  RpimO    PCV   PCV0")){ 

            n=0;
            for(j=i; j<nline-1; j++){
                if(strlen(line[j])<20  )continue;
                rid_of_front_end_space(line[j]);
                if(n>=NBIN){
                    n=0;
                    break;
                }
                if(!strncmp(line[j], "Overall", 7) && strstr_space(line[j+1],"Nmeas    Nref  Ncent")){
                    
                    k=sscanf(line[j]+8,"%d%d%d %f %*s %f %*s%*s%*s%*s %f %f %*s %f %*s %f",
                              &n1, &n2, &n3,&a1, &a2, &a3, &a4, &a5, &a6);
                    
                    if(k==9){
                        reflns.pdbx_number_measured_all=n1;
                        reflns.number_all=n2;
                        reflns.number_obs=n2;
                        reflns.pdbx_number_centric=n3;
                        reflns.percent_possible_obs=a1;
                        reflns.pdbx_redundancy=a2;
                        reflns.pdbx_Rmerge_I_obs=a3;
                        reflns.pdbx_Rrim_I_all=a3;
                        
                        reflns.pdbx_Rsym_value=a4;
                        reflns.pdbx_Rpim_I_all=a5;
                        reflns.pdbx_PCV_mean=a6;
                    }
                    i=j+1;
                    break;
                }
                
                k=sscanf(line[j],"%*s%*s %f %d%d%d %f %*s %f %*s%*s%*s%*s %f %f %*s %f %*s %f",
                         &a0,&n1, &n2, &n3,&a1, &a2, &a3, &a4, &a5, &a6);

                if(k==10){
                    if(n==0) reflns_shell.d_res_low[0] = reflns.d_resolution_low;
                    if(n>0) reflns_shell.d_res_low[n] = reflns_shell.d_res_high[n-1];
                
                    reflns_shell.d_res_high[n] = a0 ;
                    reflns_shell.number_measured_all[n] = n1 ;
                    reflns_shell.number_unique_obs[n] = n2;
                    reflns_shell.pdbx_number_centric[n] = n3;
                    reflns_shell.percent_possible_all[n] = a1;
                    reflns_shell.pdbx_redundancy[n] =a2 ;
                    reflns_shell.pdbx_Rrim_I_all[n] = a3;

                    reflns_shell.pdbx_Rsym_value[n] = a4;
                    reflns_shell.pdbx_Rpim_I_all[n] = a5;
                    reflns_shell.pdbx_PCV_mean[n] = a6;

                    n++;
                }
                
            }
            nset2=n;  
        }else if(strstr_space(line[i],"Summary data for Project: ")){
            for(j=i; j<nline-1; j++){
                rid_of_front_end_space(line[j]);
                if(strstr(line[j], "==============")) break;
                
                if(strstr_space(line[j], "Low resolution limit")){
                    reflns.pdbx_resolution_low_scale = get_float_after_id(line[j], "limit");
                    
                }else if(strstr_space(line[j], "High resolution limit")){
                    reflns.pdbx_resolution_high_scale = get_float_after_id(line[j], "limit");
                    
                }else if(!strncmp(line[j],"Rmerge ", 7)){
                    reflns.pdbx_Rmerge_I_obs = get_float_after_id(line[j], "Rmerge");
                
                }else if(strstr_space(line[j],"Rmeas (all I+ & I-)")){
                    reflns.pdbx_Rrim_I_all = get_float_after_id(line[j], ")");
                
                }else if(strstr_space(line[j],"Total number of observations")){
                    reflns.pdbx_number_measured_all = get_int_after_id(line[j], "observations");
                
                }else if(strstr_space(line[j],"Total number unique")){
                    reflns.number_obs = get_int_after_id(line[j], "unique");
                
                 }else if(strstr_space(line[j],"Mean(I)/sd(I)")){
                    reflns.pdbx_netI_over_sigmaI = get_float_after_id(line[j], "sd(I)");
                
                }else if(!strncmp(line[j],"Completeness", 12)){
                    reflns.percent_possible_obs = get_float_after_id(line[j], "Completeness");
                    
                }else if(!strncmp(line[j],"Multiplicity", 12)){
                    reflns.pdbx_redundancy = get_float_after_id(line[j], "Multiplicity");
                    
                }
            }
        }else if(strstr_space(line[i],"Average mosaicity:")){
            exptl_crystal.pdbx_mosaicity  = get_float_after_id(line[i], ":");
            
        }else if(pattern5(line[i],"Wavelength:", ".", "Cell:", ".", ".")){
            diffrn_radiation_wavelength.wavelength = get_float_after_id(line[i], "ength:");
            
        }
        
                 
    }

    if(nset1 >0 && nset2 >0 && nset1== nset2){
        reflns_shell.nbin=nset1;
    }else{
        reflns_shell.nbin=0;
        printf("Note! Only partial data was extracted from (%s)! Trying AIMLESS..\n", inpfile);
        extract_AIMLESS_log(inpfile);
        
//        printf("Warning! Only partial data was extracted from (%s)!\n", inpfile);
//        printf("nset1=%d :: nset2=%d\n", nset1 , nset2);
    }
    
}

void extract_AIMLESS_log(char *inpfile )
/* Extracting statistics from LOG files of AIMLESS
*/  
{
    char tmp[512], **line;
    int i,j=0, k=0, nline;
    float a1, a2, a3, a4, a5, a6;
    

    line=get_lines_from_file(inpfile, &nline);

    for(i=0; i<nline-1; i++){
        if (strstr_space(line[i], " * Cell Dimensions :")){
            k=sscanf(line[i+1],"%f%f%f%f%f%f", &a1, &a2, &a3, &a4, &a5, &a6);
            if( k==6 ){ add_value2cell(a1,a2,a3,a4,a5,a6);}
            
        }else if(pattern5(line[i],"Space group =", "'", "'", "(number", ")") ){
            strcpy(tmp, get_substr(line[i],"Space group =", "(number"));
            char2space(tmp, '\'');
            if(strlen(tmp)>0) strcpy(symmetry.space_group_name_H_M, tmp);
            strcpy(tmp, "");
            strcpy(tmp, get_substr(line[i],"number", ")"));
            if(strlen(tmp)>0)symmetry.Int_Tables_number = atoi(tmp);


        }else if(pattern5(line[i],"Average mosaicity:", ".", "", "", "") ){
            exptl_crystal.pdbx_mosaicity  = get_float_after_id(line[i], ":"); 

        }else if(strstr(line[i],"Wavelength: ") ){
            diffrn_radiation_wavelength.wavelength=get_float_after_id(line[i], "Wavelength:");

        }else if(pattern5(line[i],"  Wavelength ", ".", " A", "", "")  ){
            diffrn_radiation_wavelength.wavelength=get_float_after_id(line[i], "Wavelength");
            
            
        }else if(strstr(line[i],"Number of rejected outliers       " )){
            reflns.pdbx_scaling_rejects=get_int_after_id(line[i], "outliers");
            
//        }else if(strstr_space(line[i], "Summary data for ") && i<nline-1 &&
//                 strstr_space(line[i+1], "verall  InnerShell  OuterShell")){
            
        }else if(strstr_space(line[i], "verall  InnerShell  OuterShell")){
            
            for(j=i+1; j<nline-1; j++){
                if (strstr(line[j], "--------------------------")) continue;
                rid_of_front_end_space(line[j]);
                    
                k=sscanf(line[j]+37,"%f%f%f " , &a1,&a2,&a3);
                if (k==3 && strstr_space(line[j], "Low resolution limit")){
                    reflns.d_resolution_low = a1;
                    reflns_shell.d_res_low[1] = a2 ;
                    reflns_shell.d_res_low[0] = a3 ;
                    
                }else if (k==3 && strstr_space(line[j], "High resolution limit")){
                    reflns.d_resolution_high = a1;
                    reflns_shell.d_res_high[1] = a2 ;
                    reflns_shell.d_res_high[0] = a3 ;
                    reflns_shell.nbin=2;
                    
                }else if (k==3 && strstr_space(line[j], "Rmerge  (within I+/I-)") ){
                    reflns.pdbx_Rmerge_I_anomalous = a1;
                    reflns_shell.pdbx_Rmerge_I_anomalous[1] = a2;
                    reflns_shell.pdbx_Rmerge_I_anomalous[0] = a3;
                    
                }else if (k==3 && (strstr_space(line[j], "Rmerge  (all I+ and I-)") ||
                                   strstr_space(line[j], "Rmerge  (all I+ & I-)") ||
                                   strstr(line[j], "Rmerge    "))){
                    reflns.pdbx_Rmerge_I_obs = a1;
                    reflns_shell.Rmerge_I_obs[1] = a2;
                    reflns_shell.Rmerge_I_obs[0] = a3;
                    
                }else if (k==3 && strstr_space(line[j], "Rmeas (all I+ & I-)")){
                    reflns.pdbx_Rrim_I_all=a1;
                    reflns_shell.pdbx_Rrim_I_all[1] = a2;
                    reflns_shell.pdbx_Rrim_I_all[0] = a3;
                    
                }else if (k==3 && strstr_space(line[j], "Rpim (all I+ & I-)")){
                    reflns.pdbx_Rpim_I_all=a1;
                    reflns_shell.pdbx_Rpim_I_all[1] = a2;
                    reflns_shell.pdbx_Rpim_I_all[0] = a3;
                   
                }else if (k==3 && strstr_space(line[j], "Total number of observations ")){
                    reflns.pdbx_number_measured_all = int(a1);
                    reflns_shell.number_measured_all[1] = int(a2) ;
                    reflns_shell.number_measured_all[0] = int(a3) ;
                    
                }else if (k==3 && strstr_space(line[j], "Total number unique ")){
                    reflns.number_obs = int(a1);
                    reflns_shell.number_unique_obs[1] = int(a2);
                    reflns_shell.number_unique_obs[0] = int(a3);
                    
                }else if (k==3 && (strstr_space(line[j], "Mean((I)/sd(I)) ") || strstr_space(line[j], "Mean(I)/sd(I) ") )){
                    reflns.pdbx_netI_over_sigmaI = a1;
                    reflns_shell.pdbx_netI_over_sigmaI_obs[1]= a2;
                    reflns_shell.pdbx_netI_over_sigmaI_obs[0]= a3;
                    
                }else if (k==3 && (strstr_space(line[j], "Mn(I) half-set correlation CC(1/2)")||strstr(line[j],"CC(1/2) "))){
                    reflns.pdbx_CC_half = a1;
                    reflns_shell.pdbx_CC_half[1]= a2;
                    reflns_shell.pdbx_CC_half[0]= a3;
                    
                    
                }else if (k==3 && strstr(line[j], "Completeness")){
                    reflns.percent_possible_obs=a1;
                    reflns_shell.percent_possible_all[1] = a2;
                    reflns_shell.percent_possible_all[0] = a3;

                }else if (k==3 && strstr(line[j], "Multiplicity")){
                    reflns.pdbx_redundancy = a1;
                    reflns_shell.pdbx_redundancy[1] =a2 ;
                    reflns_shell.pdbx_redundancy[0] =a3 ;
                    
                }else if (strstr_space(line[j], "Estimates of resolution limits:")) break;
            }
            
        }
        
                 
    }
//    reflns_shell.nbin=2;
   
}

void extract_XIA2_log(char *inpfile )
/* Extracting statistics from LOG files of XIA2
*/  
{
    char tmp[512], **line;
    int i, nline;
    float a1, a2, a3, a4, a5, a6;
    

    line=get_lines_from_file(inpfile, &nline);

    for(i=0; i<nline-2; i++){
        if (strstr(line[i], "Unit cell:") ){
            if(sscanf(line[i+1],"%f%f%f", &a1, &a2, &a3)==3 &&
               sscanf(line[i+2],"%f%f%f", &a4, &a5, &a6)==3){
                add_value2cell(a1,a2,a3,a4,a5,a6);
            }
        }else if(strstr(line[i],"Assuming spacegroup:") ){
            strcpy(tmp, get_substr(line[i],"spacegroup:", "\n"));

        }else if(pattern5(line[i],"Mosaic spread:", ".", "<", ".", "<") ){
            exptl_crystal.pdbx_mosaicity  = get_float_after_id(line[i], "<"); 

        }else if(strstr(line[i],"Wavelength ") ){
            diffrn_radiation_wavelength.wavelength=get_float_after_id(line[i], "Wavelength");

            
        }else if(strstr(line[i],"Number of rejected outliers       " )){
            reflns.pdbx_scaling_rejects=get_int_after_id(line[i], "outliers");
            
        }else if(strstr(line[i], "High resolution limit") &&
                 sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.d_resolution_high = a1;
            reflns_shell.d_res_high[1] = a2 ;
            reflns_shell.d_res_high[0] = a3 ;
            reflns_shell.nbin=2;
            
        }else if(strstr(line[i], "Low resolution limit") &&
                 sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.d_resolution_low = a1;
            reflns_shell.d_res_low[1] = a2 ;
            reflns_shell.d_res_low[0] = a3 ;
                    
        }else if (strstr(line[i], "Completeness  ") &&
                  sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.percent_possible_obs=a1;
            reflns_shell.percent_possible_all[1] = a2;
            reflns_shell.percent_possible_all[0] = a3;

        }else if (strstr(line[i], "Multiplicity  ") &&
                  sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.pdbx_redundancy = a1;
            reflns_shell.pdbx_redundancy[1] =a2 ;
            reflns_shell.pdbx_redundancy[0] =a3 ;

        }else if (strstr(line[i], "I/sigma  ") &&
                  sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.pdbx_netI_over_sigmaI = a1;
            reflns_shell.meanI_over_sigI_obs[1]= a2;
            reflns_shell.meanI_over_sigI_obs[0]= a3;

        }else if (strstr(line[i], "Rmerge ") &&
                  sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.pdbx_Rmerge_I_obs = a1;
            reflns_shell.Rmerge_I_obs[1] = a2;
            reflns_shell.Rmerge_I_obs[0] = a3;

        }else if (strstr(line[i], "Rmeas(I) ") &&
                  sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.pdbx_Rrim_I_all=a1;
            reflns_shell.pdbx_Rrim_I_all[1] = a2;
            reflns_shell.pdbx_Rrim_I_all[0] = a3;

        }else if (strstr(line[i], "Rpim(I)  ") &&
                  sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.pdbx_Rpim_I_all=a1;
            reflns_shell.pdbx_Rpim_I_all[1] = a2;
            reflns_shell.pdbx_Rpim_I_all[0] = a3;

        }else if (strstr(line[i], "Wilson B factor  ") &&
                  sscanf(line[i]+25,"%f", &a1)==1){
            reflns.B_iso_Wilson_estimate = get_float_after_id(line[i], "factor");

        }else if (strstr(line[i], "Total observations  ") &&
                  sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.pdbx_number_measured_all = int(a1);
            reflns_shell.number_measured_all[1] = int(a2) ;
            reflns_shell.number_measured_all[0] = int(a3) ;

        }else if (strstr(line[i], "Total unique  ") &&
                  sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.number_obs = int(a1);
            reflns_shell.number_unique_obs[1] = int(a2);
            reflns_shell.number_unique_obs[0] = int(a3);

        }else if (strstr(line[i], "Rmeas(I+/-) ") &&
                  sscanf(line[i]+25,"%f%f%f", &a1, &a2, &a3)==3){
            reflns.pdbx_Rmerge_I_anomalous = a1;
            reflns_shell.pdbx_Rmerge_I_anomalous[1] = a2;
            reflns_shell.pdbx_Rmerge_I_anomalous[0] = a3;
        }
    }
//    reflns_shell.nbin=2;
   
}

void extract_3DSCALE_log(char *inpfile )
// Extracting statistics from LOG files of 3DSCALE
{
    float junk, resl, resh, rim_a, pim_a, cch_a, rim, pim, cch;
    float redun_a, redun, rsym_a, rsym, isig_a, isig, chi2_a, comp_a, comp;
    float a1, a2, a3, a4, a5, a6;
    
    char tmp[512], **line;
    int k, m, i,i1,i2,n, n1=0, nline, na, na_junk;
    

    line=get_lines_from_file(inpfile, &nline);

    for(i=0; i<nline-2; i++){
        if (pattern5(line[i], "Unit cell",": ",".",".",".") ){
            strcpy(tmp, strstr(line[i],":")+1);
            k=sscanf(tmp,"%f%f%f%f%f%f", &a1, &a2, &a3, &a4, &a5, &a6);
            if(k==6){ add_value2cell(a1,a2,a3,a4,a5,a6);}

        }else if(pattern5(line[i], "Space Group:  ", "","","","") ){
            k=sscanf(strstr(line[i],":")+1,"%d%s", &n, tmp );
            if(k==2){
                symmetry.Int_Tables_number = n;
                strcpy(symmetry.space_group_name_H_M, strstr(strstr(line[i],":")+1, tmp));
                rid_of_front_end_space(symmetry.space_group_name_H_M);
            }
            
        }else if(pattern5(line[i], " Overall Resolution Cutoff ", ": ","","","") ){
            strcpy(tmp, strstr(line[i],":")+1);
            k=sscanf(tmp,"%f%f", &a1, &a2);
            if(k==2){
                reflns.d_resolution_low= a1;
                reflns.d_resolution_high= a2;
            }
        }else if(strstr_space(line[i], "     rejected as outliers") ){

            k=sscanf(line[i],"%d %d", &i1, &i2);
            if(k==2){
                reflns.pdbx_scaling_rejects=i2;
            }
            
        }else if(strstr(line[i], "     low  I/SigI cutoffs    ") ){
            reflns.observed_criterion_sigma_I=get_float_after_id(line[i],"cutoffs ");
            
        }else if(strstr_space(line[i],"Reso.Shell nRefObs nRefExp       Rmeas           Rpim        CC1/2         CC*")
                 && strstr_space(line[i+1],"--all ----all  ---all---shell  ---all---shell --all-shell")){
            n=0;
            for(k=i+2; k<nline-2; k++){
                m=sscanf(line[k],"%f%s%f %d%d %f%f %f%f %f%f ",&resl,tmp,&resh, &na,&na_junk, &rim_a,&rim,  &pim_a,&pim, &cch_a,&cch);
                if (n>0 && m!=11) break;

                if (m==11 && strstr(tmp,"-")){
                    reflns_shell.d_res_low[n]=resl;
                    reflns_shell.d_res_high[n]=resh;
                        //       reflns_shell.number_unique_obs[n]=na;
                    reflns_shell.pdbx_Rrim_I_all[n]=rim;
                    reflns_shell.pdbx_Rpim_I_all[n]=pim;
                    reflns_shell.pdbx_CC_half[n]=cch;
                
                    reflns.number_obs=na;
                    reflns.pdbx_Rrim_I_all=rim_a;
                    reflns.pdbx_Rpim_I_all=pim_a;
                    reflns.pdbx_CC_half=cch_a;
                    n=n+1;
                }
            }

        }else if(strstr_space(line[i],"Reso.Shell     Compl%        Redund          Rsym        Rfree nRfree <i/sigi>observe  <I/sigI>merged <Chi^2>")
                 && strstr_space(line[i+1]," --all--shell ---all--shell  ---all---shell  ---all ---al")){
            
            n1=0;
            for(k=i+2; k<nline; k++){
                m=sscanf(line[k],"%f%s%f  %f%f  %f%f %f%f  %f%f  %f%f  %f%f %f",&resl,tmp,&resh,  &comp_a,&comp,
                         &redun_a,&redun,  &rsym_a, &rsym, &junk, &junk, &isig_a, &isig, &junk, &junk, &chi2_a);
                
                if (n1>0 && m!=16) break;

                if (m==16 && strstr(tmp,"-")){
                    reflns_shell.percent_possible_all[n1]=comp;
                    reflns_shell.pdbx_redundancy[n1]=redun;
                    reflns_shell.pdbx_Rsym_value[n1]=rsym;
                    reflns_shell.pdbx_netI_over_sigmaI_obs[n1]=isig;

                    reflns.percent_possible_obs=comp_a;
                    reflns.pdbx_redundancy=redun_a;
                    reflns.pdbx_Rsym_value=rsym_a;
                    reflns.pdbx_netI_over_sigmaI=isig_a;
                    reflns.pdbx_chi_squared=chi2_a;
                    n1=n1+1;  
                }
            }
        }
    }
 
    reflns.pdbx_resolution_high_scale=reflns.d_resolution_high;
    reflns.pdbx_resolution_low_scale=reflns.d_resolution_low;

    if(n>0 && n==n1){
        reflns_shell.nbin=n;
    }else{
        reflns_shell.nbin=0;
        printf("Warning! Nothing extracted from (%s)!\n", inpfile);
    }

}


void extract_XSCALE_log(char *inpfile)
// Extracting statistics from LOG files of 3DSCALE
{
    char **line, tmp[256], comp[20], rmea[20], rmg_f[20], rmg[20], cut[20];
    int k=0, n=0, i, j, nf=0, nf_u=0, nf_p=0,  nline;
    float a1, a2, a3, a4, a5, a6;

    strcpy(cut, "-3.0");
    if(strlen(DATACUT)>1) strcpy(cut,DATACUT);

    line=get_lines_from_file(inpfile, &nline);
      
//    reflns.d_resolution_low = refine.ls_d_res_low;
    for(i=0; i<nline; i++){
        if (pattern5(line[i], "UNIT_CELL_CONSTANTS=",".",".",".",".") ){
            strcpy(tmp, strstr(line[i],"=")+1);
            k=sscanf(tmp,"%f%f%f%f%f%f", &a1, &a2, &a3, &a4, &a5, &a6);
            if(k==6){ add_value2cell(a1,a2,a3,a4,a5,a6);
            }

        }else if(pattern5(line[i], "SPACE_GROUP_NUMBER=", "","","","") ){
            strcpy(tmp, strstr(line[i],"=")+1);
            k=sscanf(tmp,"%d", &n);
            if(k==1){
	      // std::cout << "Space Group Number from XSCALE log file: " << n << '\n'; 
	      symmetry.Int_Tables_number = n;
            }
        }else if(pattern5(line[i], "WILSON LINE (using all data)", "A=", "B=", "CORRELATION=", "")){
            reflns.B_iso_Wilson_estimate = get_float_after_id(line[i], "B=");

        }else if(strstr(line[i], "CHI^2-VALUE OF FIT OF CORRECTION FACTORS")){
            reflns.pdbx_chi_squared = get_float_after_id(line[i], "FACTORS");

        }else if(pattern5(line[i], "X-RAY_WAVELENGTH", "=", ".", "", "")){
            diffrn_radiation_wavelength.wavelength = get_float_after_id(line[i], "=");

        }else if(pattern5(line[i], " OUT OF ", " REFLECTIONS REJECTED", "", "", "")){
            if (sscanf(line[i], "%d", &k)==1){ reflns.pdbx_scaling_rejects =k;}
            
        }else if(pattern5(line[i], "RESOLUTION RANGE", "Sigma", "", "", "") &&
                 pattern5(line[i+1], "observed", "expected", "", "", "") ){
            
            if (sscanf(line[i+2], "%f", &a1)==1){ reflns.d_resolution_low =a1;}
            
        }else if(strstr_space(line[i], "XSCALE DATA PROCESSING                        |")){
            n=0;
            for(j=i+1; j<nline-1; j++){
                if (pattern5(line[j], "Resolution (overall) [A] ", "|", ".", "-",".")) {
                    strcpy(tmp, strstr(line[j],"|")+1 );
                    char2space(tmp, '-');
                    if (sscanf(tmp, "%f %f", &a1, &a2)==2){
                        reflns.d_resolution_low =a1;
                        reflns.d_resolution_high =a2;
                    }
                }else if ( pattern5(line[j], "Resolution (highest shell) [A]", "|", ".", "-",".")) {
                    strcpy(tmp, strstr(line[j],"|")+1 );
                    char2space(tmp, '-');
                    if (sscanf(tmp, "%f %f", &a1, &a2)==2){
                        reflns_shell.d_res_low[0] =a1;
                        reflns_shell.d_res_high[0] =a2;
                        n=1;
                    }
                }else if ( pattern5(line[j], "Crystal mosaicity [deg]   ", "|", ".", "","")) {
                     exptl_crystal.pdbx_mosaicity=get_float_after_id(line[j], "|");
                }else if ( pattern5(line[j], "Rsym (overall) [%]", "|", ".", "","")) {
                     reflns.pdbx_Rmerge_I_obs=0.01*get_float_after_id(line[j], "|");
                }else if ( pattern5(line[j], "Rsym (highest shell) [%]", "|", ".", "","")) {
                     reflns_shell.Rmerge_I_obs[0]=0.01*get_float_after_id(line[j], "|");

                }else if ( pattern5(line[j], "Rmeas/Rrim (overall) [%]", "|", ".", "","")) {
                     reflns.pdbx_Rrim_I_all=0.01*get_float_after_id(line[j], "|");
                }else if ( pattern5(line[j], "Rmeas/Rrim (highest shell) [%]", "|", ".", "","")) {
                     reflns_shell.pdbx_Rrim_I_all[0]=0.01*get_float_after_id(line[j], "|");

                }else if ( pattern5(line[j], "RmrgdF/Rpim (overall) [%]", "|", ".", "","")) {
                     reflns.pdbx_Rpim_I_all=0.01*get_float_after_id(line[j], "|");
                }else if ( pattern5(line[j], "RmrgdF/Rpim (highest shell) [%]", "|", ".", "","")) {
                     reflns_shell.pdbx_Rpim_I_all[0]=0.01*get_float_after_id(line[j], "|");

                }else if ( pattern5(line[j], "I/sig(I) (overall) ", "|", ".", "","")) {
                     reflns.pdbx_netI_over_sigmaI=get_float_after_id(line[j], "|");
                }else if ( pattern5(line[j], "I/sig(I) (highest shell) ", "|", ".", "","")) {
                     reflns_shell.meanI_over_sigI_obs[0]=get_float_after_id(line[j], "|");

                }else if ( pattern5(line[j], "Completeness (overall) ", "|", ".", "","")) {
                     reflns.percent_possible_obs=get_float_after_id(line[j], "|");
                }else if ( pattern5(line[j], "Completeness (highest shell) ", "|", ".", "","")) {
                     reflns_shell.percent_possible_all[0]=get_float_after_id(line[j], "|");

                }else if ( pattern5(line[j], "Completeness (overall) ", "|", ".", "","")) {
                     reflns.percent_possible_obs=get_float_after_id(line[j], "|");
                }else if ( pattern5(line[j], "Completeness (highest shell) ", "|", ".", "","")) {
                     reflns_shell.percent_possible_all[0]=get_float_after_id(line[j], "|");

                }else if ( pattern5(line[j], "Redundancy/Multiplicity (overall) ", "|", ".", "","")) {
                     reflns.pdbx_redundancy=get_float_after_id(line[j], "|");
                }else if ( pattern5(line[j], "Redundancy/Multiplicity (highest shell) ", "|", ".", "","")) {
                     reflns_shell.pdbx_redundancy[0]=get_float_after_id(line[j], "|");

                }else if ( pattern5(line[j], "Number of observed reflections (overall)", "|", "", "","")) {
                     reflns.pdbx_number_measured_all=get_int_after_id(line[j], "|");

                }else if ( pattern5(line[j], "Number of unique reflections (overall)", "|", "", "","")) {
                     reflns.number_obs=get_int_after_id(line[j], "|");

                }else if ( pattern5(line[j], "Wilson Bfactor [A2] ", "|", ".", "","")) {
                     reflns.B_iso_Wilson_estimate=get_float_after_id(line[j], "|");
                }else if ( pattern5(line[j], "Negative intensity cutoff [I/sig(I)]", "|", ".", "","")) {
                     reflns.observed_criterion_sigma_I = get_float_after_id(line[j], "|");
                }
            }



        }else if(pattern5(line[i], "SUBSET OF INTENSITY DATA WITH SIGNAL/NOISE >=", cut, "AS FUNCTION OF RESOLUTION", "", "") ){
            n=0;
            for(j=i+3; j<nline-1; j++){
                k=sscanf(line[j],"%f %d %d %d %s %s %*s %*s %f %s %s ", &a1, &nf, &nf_u, &nf_p, comp, rmg, &a2, rmea, rmg_f);
                if(nf==0 && comp[0]=='-' && rmg[0]=='-') continue;
                if(pattern5(line[j],"total ", ".", "%", "%", "%")){
                    k=sscanf(line[j],"%*s %d %d %d %s %s %*s %*s %f %s %s ", &nf, &nf_u, &nf_p, comp, rmg, &a2, rmea, rmg_f);
                    if(k==8){
                        reflns.observed_criterion_sigma_I = atof(cut);
                        reflns.d_resolution_high=reflns_shell.d_res_high[n-1];
                        reflns.pdbx_number_measured_all=nf;
                        reflns.number_obs=nf_u;
                        reflns.pdbx_redundancy=float(nf)/nf_u;
                        if (strchr(comp, '%')){
                            char2space(comp, '%');
                            reflns.percent_possible_obs=atof(comp);
                        }
                        if (strchr(rmg, '%')){
                            char2space(rmg, '%');
                            reflns.pdbx_Rmerge_I_obs=0.01*atof(rmg);
                        }

                        reflns.pdbx_netI_over_sigmaI=a2;
                        if (strchr(rmea, '%')){                        
                            char2space(rmea, '%');
                            reflns.pdbx_Rrim_I_all=0.01*atof(rmea);
                        }
                        if (strchr(rmg_f, '*')){
                            char2space(rmg_f, '*');
                            reflns.pdbx_CC_half=0.01*atof(rmg_f);
                        }else if (strchr(rmg_f, '%')){
                            char2space(rmg_f, '%');
                            reflns.Rmerge_F_obs=0.01*atof(rmg_f);
                        }
                    }
                    break;
                    
                }else if( k==9 && pattern5(line[j], ".", "%", "%", "%", ".")){
                    reflns_shell.d_res_high[n]=a1;
                    if(n>0) reflns_shell.d_res_low[n] = reflns_shell.d_res_high[n-1];
                    reflns_shell.number_measured_obs[n]=nf;
                    reflns_shell.number_unique_obs[n]=nf_u;
                    reflns_shell.number_possible[n]=nf_p;
                    reflns_shell.pdbx_redundancy[n]=float(nf)/nf_u;
                    char2space(comp, '%');
                    reflns_shell.percent_possible_all[n]=atof(comp);

                    char2space(rmg, '%');
                    reflns_shell.Rmerge_I_obs[n]=0.01*atof(rmg);
                    
                    reflns_shell.meanI_over_sigI_obs[n]=a2;

                    char2space(rmea, '%');
                    reflns_shell.pdbx_Rrim_I_all[n]=0.01*atof(rmea);
                    
                    if (strchr(rmg_f, '*')){
                        char2space(rmg_f, '*');
                        reflns_shell.pdbx_CC_half[n]=0.01*atof(rmg_f);
                    }else if (strchr(rmg_f, '%')){
                        char2space(rmg_f, '%');
                        reflns_shell.Rmerge_F_obs[n]=0.01*atof(rmg_f);
                    }

                    n++;
                }
            }
        }
    }
    reflns_shell.d_res_low[0]=reflns.d_resolution_low;
    reflns.pdbx_resolution_high_scale=reflns.d_resolution_high;
    reflns.pdbx_resolution_low_scale=reflns.d_resolution_low;
    
    if(n>0){
        reflns_shell.nbin =n;
    }else{
        reflns_shell.nbin=0;
        printf("Warning! Nothing extracted from (%s)!\n", inpfile);
    }
    if(nline>0) free_memory_2d(line, nline);
    
}


void extract_PROSCALE_log(char *inpfile)
// Extracting statistics from LOG files of PROSCALE
{
    char tmp[512], **lines;
    int i,ii,k=0,j=0, n=0, nline=0;
    int n1,n2,n3,n4; 
    float a1, a2, a3, a4, a5, a6, b[20];

    lines=get_lines_from_file(inpfile, &nline);

    for(i=0; i<nline; i++){
        if(strstr_space(lines[i], "Space Group:")){
            n=get_int_after_id(lines[i]," Group:");
            if(n>0) symmetry.Int_Tables_number = n;
            get_string_with_length(lines[i],18, 35, tmp);
            rid_of_front_end_space(tmp);
            if(strlen(tmp)>0) strcpy(symmetry.space_group_name_H_M, tmp);
            
        }else if(strstr_space(lines[i], "<<< Unit cell >>>:")){
            strcpy(tmp, strstr(lines[i],">>>:")+5);
            k=sscanf(tmp,"%f%f%f%f%f%f", &a1, &a2, &a3, &a4, &a5, &a6);
            if(k==6){ add_value2cell(a1,a2,a3,a4,a5,a6);}
            
        }else if(strstr_space(lines[i], "<<< Overall Resolution Cutoff >>>:")){
            strcpy(tmp, strstr(lines[i],">>>:")+5);
            k=sscanf(tmp,"%f%f", &a1, &a2);
            if(k==2){
                reflns.d_resolution_high = a2;
                reflns.d_resolution_low = a1;
            }

        }else if(strstr_space(lines[i], "<<< Overall I/SigI Cutoff >>>:")){
            strcpy(tmp, strstr(lines[i],">>>:")+5);
            k=sscanf(tmp,"%f%f", &a1, &a2);
            if(k==2){
                reflns.observed_criterion_I_min = a1;
                reflns.observed_criterion_I_max = a2;
            }

        }else if(strstr_space(lines[i],"Observed      Unique    Observed      Unique ")){
            if(i<nline-2 && strstr_space(lines[i+1],"total reflections")){
                if(sscanf(lines[i+1],"%d%d%d%d", &n1, &n2, &n3, &n4)==4){
                    reflns.pdbx_number_measured_all  = n3;
                    reflns.number_obs  = n4;
                }
            }

        }else if(strstr_space(lines[i],"    rejected as outliers")){
            n1 = get_int_by_length(lines[i], 4, 14);
            n2 = get_int_by_length(lines[i], 28, 37);
            if(n1>=0 && n2>=0){
                reflns.pdbx_scaling_rejects = n2;
            }
        }else if(strstr_space(lines[i],"Res.Shell nRefObs nRefExp Compl% Redund    Rsym---Shell")){
            n=0;
            for(j=i; j<nline;j++){
                k=sscanf(lines[j], "%s%f %*s%*s %f%f%f%f %*s%*s %f%f%f%f%f",
                         tmp, &b[0],&b[1],&b[2],&b[3],&b[4],&b[5],&b[6],&b[7],&b[8],&b[9]);


                if( k==11 && !strcmp_case(tmp, "to") ){

                    reflns_shell.d_res_high[n]=b[0];
                    reflns.percent_possible_obs = b[1];
                    reflns.pdbx_redundancy = b[2];
                    reflns.pdbx_Rsym_value =  b[3];
                    reflns_shell.pdbx_Rsym_value[n] =  b[4];
//                    reflns.pdbx_netI_over_av_sigmaI = b[5];
                    reflns.pdbx_netI_over_sigmaI = b[5];
                    reflns_shell.meanI_over_sigI_obs[n] = b[6];
                    reflns.pdbx_chi_squared  = b[7];
                    reflns.pdbx_netI_over_av_sigmaI_anomalous  = b[8];
                    reflns_shell.pdbx_meanI_over_sigI_anomalous[n]  = b[9];

                    n++;
                }
                if(n>1 && k !=11){
                    reflns_shell.d_res_low[0]=reflns.d_resolution_low;
                    for(ii=0; ii<n-1; ii++){
                        reflns_shell.d_res_low[ii+1] = reflns_shell.d_res_high[ii];
                    }
                    i=j;

                    break;
                }
            }

        }
    }
    reflns.pdbx_resolution_high_scale=reflns.d_resolution_high;
    reflns.pdbx_resolution_low_scale=reflns.d_resolution_low;

    if(n>0 ){
        reflns_shell.nbin=n;
    }else{
        reflns_shell.nbin=0;
        printf("Warning! Nothing extracted from (%s)!\n", inpfile);
    }

}


void extract_XPREP_log(char *inpfile)
// Extracting statistics from LOG files of XPREP
{
    char tmp[256], **lines;
    int i,k=0,j=0, n=0, nline=0;
    int n1,n2,n3,n4, rpim=0;
    float a1, a2, a3, a4, a5, a6, b[20];

    lines=get_lines_from_file(inpfile, &nline);

    for(i=0; i<nline-1; i++){
        if(strstr_space(lines[i], "Original cell in Angstroms and degrees:")){
            k=sscanf(lines[i+1],"%f%f%f%f%f%f", &a1, &a2, &a3, &a4, &a5, &a6);
            if(k==6){ add_value2cell(a1,a2,a3,a4,a5,a6);}

        }else if(strstr_space(lines[i], "Reflections read from file")){
            k=sscanf(lines[i],"%d", &n1);
            if(k==1){
                reflns.pdbx_number_measured_all = n1;
            }
            
        }else if(strstr_space(lines[i], "<<< Overall I/SigI Cutoff >>>:")){
            strcpy(tmp, strstr(lines[i],">>>:")+5);
            k=sscanf(tmp,"%f%f", &a1, &a2);
            if(k==2){
                reflns.observed_criterion_I_min = a1;
                reflns.observed_criterion_I_max = a2;



                reflns.d_resolution_high = a2;
                reflns.d_resolution_low = a1;
            }
            
        }else if(strstr_space(lines[i],"Observed      Unique    Observed      Unique ")){
            if(i<nline-2 && strstr_space(lines[i+1],"total reflections")){
                if(sscanf(lines[i+1],"%d%d%d%d", &n1, &n2, &n3, &n4)==4){
                    reflns.pdbx_number_measured_all  = n3;
                    reflns.number_obs  = n4;
                }
            }
            
        }else if(strstr_space(lines[i],"    rejected as outliers")){
            n1 = get_int_by_length(lines[i], 4, 14);
            n2 = get_int_by_length(lines[i], 28, 37);
            if(n1>=0 && n2>=0){
                reflns.pdbx_scaling_rejects = n2;
            }
        }else if(strstr_space(lines[i],"Resolution  #Data #Theory %Complete Redundancy Mean I Mean I/s R(int)")){
            if (strstr(lines[i]+65, "Rpim"))rpim=1;
            n=0;
            for(j=i+1; j<nline-3;j++){
                if(strstr(lines[j],"-------------------------------")){
                    if(pattern5(lines[j+2],"Inf", " - ", "." , "." , "." )){
                        k=sscanf(lines[j+2], "%*s%*s %f %d%d %f%f%f%f%f%f",
                                 &b[0], &n1, &n2, &b[1],&b[2],&b[3],&b[4],&b[5],&b[6]);
                        if (k==9){
                            reflns.d_resolution_high=b[0];
                            reflns.number_obs=n1;
                            reflns.percent_possible_obs = b[1];
                            reflns.pdbx_redundancy = b[2];
                            reflns.pdbx_average_I_obs =b[3];
                            reflns.pdbx_netI_over_sigmaI = b[4];
                            reflns.pdbx_Rmerge_I_obs =  b[5];
                            if (rpim) reflns.pdbx_Rpim_I_all=b[6];
                        }
                        
                    }
                    if(pattern5(lines[j+3],"Merged", "lowest resolution =", "Angstroms" , "" , "" )){
                        reflns.d_resolution_low = get_float_after_id(lines[j+3], "ion =");
                        reflns_shell.d_res_low[0]=reflns.d_resolution_low;
                    }
                    i=j+1;
                    break;
                }else if (pattern5(lines[j], " - ", ".", ".", ".", ".")){
                    k=sscanf(lines[j], "%s %*s %f %d%d %f%f%f%f%f%f",tmp,
                             &b[0], &n1, &n2, &b[1],&b[2],&b[3],&b[4],&b[5],&b[6]);
                    if(k==10){
                        reflns_shell.d_res_high[n]=b[0];
                        
                        if(n>0) reflns_shell.d_res_low[n] = reflns_shell.d_res_high[n-1];
                        reflns_shell.number_unique_obs[n]=n1;
                        reflns_shell.number_possible[n]=n2;
                        reflns_shell.percent_possible_all[n]=b[1];
                        reflns_shell.pdbx_redundancy[n]=b[2];
                    
                        reflns_shell.pdbx_netI_over_sigmaI_obs[n]=b[4];
                    
                        reflns_shell.Rmerge_I_obs[n]=b[5];
                        
                        if (rpim) reflns_shell.pdbx_Rpim_I_all[n]=b[6];
                        n++;
                        reflns_shell.nbin=n;
                    }
                }
            }
            
        }
    }
    reflns_shell.d_res_low[0]=reflns.d_resolution_low;
    if(reflns_shell.nbin==0)
        printf("Warning! Nothing extracted from (%s)!\n", inpfile);
    
}

void I_over_sigmaI(char *inpfile)
{
    char  str[100];
    int  m, h, k, l, hh=0, kk=0, ll=0, nrefl=0, len, nshell[NBIN];
    float i=1, si=1, ii=1, sii=1, i_si=1,sum_i_si=0, I_sigmaI_av[NBIN];
    float cel[6],rcell[6],ah, ak, al, resolution;
    FILE *fp=NULL;

    
    if((fp=fopen(inpfile, "r")) == NULL)
        open_file_error(inpfile, "I_over_sigmaI");
        
    for(m=0; m<6; m++) {
        cel[m]=INI;
    }
    
    if(reflns_shell.nbin<=0) return;
    printf("Number of bins=%d\n", reflns_shell.nbin);
    
    for(m=0; m<reflns_shell.nbin; m++){
        I_sigmaI_av[m] = 0;
        nshell[m]  =   0;
    }
    
    while (fgets(str, sizeof str, fp) !=NULL){
        if(strlen(str)>61 && pattern5(str,".",".",".",".",".")){
            sscanf(str, "%f%f%f%f%f%f",
                   &cel[0], &cel[1], &cel[2],
                   &cel[3], &cel[4], &cel[5]);
            break;
        }  
    }
    if((cel[0]<=0 || cel[1]<=0) && cell.length_a>0){
        cel[0] = cell.length_a;
        cel[1] = cell.length_b;
        cel[2] = cell.length_c;
        cel[3] = cell.angle_alpha;
        cel[4] = cell.angle_beta;
        cel[5] = cell.angle_gamma;
    }
    
    if(cel[0]<=0 || cel[1]<=0 || cel[5]<=0) return;
    calc_recip_cell(cel, rcell);

    
    while (fgets(str, sizeof str, fp) !=NULL){
        rid_of_end_space(str);
        len=strlen(str);

        h = get_int_by_length(str, 0, 3);
        k = get_int_by_length(str, 4, 7);
        l = get_int_by_length(str, 8, 11);

        if(len==28 || len==29 ){
            i = get_float_by_length(str, 12, 19);
            si= get_float_by_length(str, 20, 27);
            
        }else if(len==35 || len==36){
            i = get_float_by_length(str, 19, 26);
            si= get_float_by_length(str, 27, 34);
            
        }else if(len==44 || len==45){
            
            i = get_float_by_length(str, 12, 19);
            si= get_float_by_length(str, 20, 27);

            hh =  -h;
            kk =  -k;
            ll =  -l;
            
            ii = get_float_by_length(str, 28, 35);
            sii= get_float_by_length(str, 36, 43);
            
        }else if(len==53 || len==54){
            i = get_float_by_length(str, 37, 44);
            si= get_float_by_length(str, 45, 52);
        }


        ah= float (h);
        ak= float (k);
        al= float (l);
        
        resolution = get_resolution(ah, ak, al, rcell);
        
        if (h!=0 || k!=0 || l!=0){
            if(si>0.0001) {
                i_si = i/si;
                sum_i_si = sum_i_si + i_si;
            }
            nrefl++;
        }
        if(hh!=0 || kk!=0 || ll!=0){
            if(sii>0.0001) {
                i_si = i/si;
                sum_i_si = sum_i_si + ii/sii;
            }
            nrefl++;
        }
        
        if(reflns_shell.nbin>0){
            for(m=0; m<reflns_shell.nbin; m++){
                if(resolution >=reflns_shell.d_res_high[m] &&
                   resolution <reflns_shell.d_res_low[m]){
                    I_sigmaI_av[m]  =  I_sigmaI_av[m] +  i_si;
                    nshell[m]++;
                    

                }
            }
        }
        
    }
    
    if(nrefl >0 && reflns_shell.nbin>0){
        for(m=0; m< reflns_shell.nbin; m++){
            reflns_shell.meanI_over_sigI_obs[m] = I_sigmaI_av[m]/nshell[m] ;
            
        }
    }
    if(nrefl >0)reflns.pdbx_netI_over_av_sigmaI = sum_i_si/float(nrefl);
 
}


float get_resolution(float h, float k, float l, float *rcell)
{

    float aa1, aa2, aa3, a2, b2, c2, resol=0, dist_sq;
    
    
    aa1 =  2 * rcell[0] * rcell[1] * cos(DEG2RAD*rcell[5]);
    aa2 =  2 * rcell[0] * rcell[2] * cos(DEG2RAD*rcell[4]);
    aa3 =  2 * rcell[1] * rcell[2] * cos(DEG2RAD*rcell[3]);

    a2 =  rcell[0]*rcell[0];
    b2 =  rcell[1]*rcell[1];
    c2 =  rcell[2]*rcell[2];

    dist_sq = h*h*a2 + k*k*b2 + l*l*c2 + h*k*aa1 + h*l*aa2 + k*l*aa3;
    if(dist_sq>0.0000001)
        resol = sqrt(1.0/dist_sq);

    return resol;
    
}
    
void calc_recip_cell(float *cell, float *rcell)
{
    float cosa,cosb, cosc,sina,sinb,sinc, cosast,cosbst,coscst,v;
    
    
    cosa = cos(DEG2RAD*cell[3]);
    cosb = cos(DEG2RAD*cell[4]);
    cosc = cos(DEG2RAD*cell[5]);
    sina = sin(DEG2RAD*cell[3]);
    sinb = sin(DEG2RAD*cell[4]);
    sinc = sin(DEG2RAD*cell[5]);
    
    v = cell[0]* cell[1] *cell[2]*
        sqrt(1. - cosa*cosa - cosb*cosb - cosc*cosc + 2.*cosa*cosb*cosc );
    
    rcell[0] =  cell[1] * cell[2]* sina / v;
    rcell[1] =  cell[0] * cell[2]* sinb / v;
    rcell[2] =  cell[0] * cell[1]* sinc / v;
    
    cosast   =  (cosb*cosc-cosa) / (sinb*sinc);
    cosbst   =  (cosa*cosc-cosb) / (sina*sinc);
    coscst   =  (cosa*cosb-cosc) / (sina*sinb);
    
    rcell[3] =  acos(cosast)/DEG2RAD;
    rcell[4] =  acos(cosbst)/DEG2RAD;
    rcell[5] =  acos(coscst)/DEG2RAD;
    
}

