/* convert a structure factor file from one format to another */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include "sf_convert.h"
#include "util.h"
#include "TableFile.h"
#include "CifFile.h"
#include "CifParserBase.h"

extern char  PDBID[80], AUDIT[512], INFO[8192], DETAIL[999];
extern int CIF_CHECK, VERB;

static int get_cifitem_list(int start, int nline, char **line, char **cif_items);

void check_data_block_name(char *file, char *block_id, char *pdbid, int num);

void reformat_sfhead(char *infile, char *outfile)
//reformat the header of infile
{
    char infile_tmp[512],  pdbid[20], str[512], tmp[256], block_id[200];
    char  **line=NULL, **refln=NULL, **item1=NULL,tmpf[256],tmpf_new[256];
    int i, j, n1=0, nline, nh=0, nb=0, nf=0, k, m, nnf;
    FILE *fw, *fout, *ftmp;
    
    strcpy(pdbid, ".");

    sprintf(infile_tmp, "%s_tmp", infile);
    precheck_sf(infile, infile_tmp);
        
    line=get_lines_from_file(infile_tmp, &nline);
    if (VERB==0) delete_file(infile_tmp);
    
    rename_cif_token(nline, line, infile_tmp);
    
    fout=fopen(outfile, "w");
    
    nb=0;
    for(i=0; i<nline; i++){
        if (strchr(line[i],'_')|| strstr(line[i]," ;")|| strchr(line[i],'#')){
            rid_of_front_space(line[i]);
        }
 
        if (!strncmp(line[i], "data_", 5)){
            strcpy(block_id,line[i]+5);
            check_data_block_name(infile, block_id, tmp, nb);
            
            if(nb==0 && strlen(tmp)==4) {strcpy(pdbid, tmp); }            
                
            nb++;
        
//            printf("block_id=%s  %d\n", block_id, nb);
            refln=new char *[nline];
            nf=0;
            for (j=i; j<nline; j++){ //one data block.
                if (strchr(line[j],'_') || strstr(line[j]," ;") || strchr(line[j],'#')){
                    rid_of_front_space(line[j]);
                }
                if ( j-i>10 &&  !strncmp(line[j], "data_", 5) ){
                    i=j-1;
                    break;
                }
                refln[nf] = new char [strlen(line[j]) +1];
                strcpy(refln[nf], line[j]);
                nf++;
            }
            if (nf<50) {
                sprintf(INFO, "Error: File (%s) has no data in this block %d.\n", infile, nb);
                pinfo(INFO,0);
                continue;
            }

            nnf=0; //number of _refln. in the block
            sprintf(tmpf, "head_part_%d", nb);
            fw=fopen(tmpf,"w");
            n1=0;    
            item1=new char *[nline];
            for(k=0; k<nf; k++){
                if (k<nf-1 && !strncmp(refln[k], "loop_", 5) && 
                     (!strncmp(refln[k+1], "_refln.", 7)||
                     !strncmp(refln[k+1], "_diffrn_refln.",14)||
                     !strncmp(refln[k+1], "_phasing_MIR_der_refln.", 23)) ){
                    n1++;                   

                    for (m=k; m<nf; m++){ //number of reflections in this block.
                        if (m-k>100 && (refln[m][0]=='_' || !strncmp(refln[m], "loop_", 5) )){
                            break;
                        }
                        item1[nnf]=new char [strlen(refln[m]) +1];
                        strcpy(item1[nnf], refln[m]);
                        nnf++;
                    }
                    k=m-1;
                }else{
                    fprintf(fw, "%s", refln[k]);
                }
            }
            fclose(fw);
            if(n1>=2){
                sprintf(INFO, "Error: File (%s) has multiple(%d) _refln loops in data block %d.\n", infile,n1, nb);
                pinfo(INFO,0);
            }
            
            sprintf(tmpf_new, "%s_head_%d",infile, nb);
            
            update_items(tmpf, tmpf_new, pdbid, nb);
            
            cifparse(tmpf_new, tmpf_new, 100) ; 
            if(CIF_CHECK==1) cif_dic_check(tmpf_new);

            
            ftmp=fopen(tmpf_new, "r");
            nh=0;
            while(fgets(str, sizeof str, ftmp)!=NULL){
                fprintf(fout, "%s", str);
                nh++;
            }
            fclose(ftmp);
            
            for (k=0; k<nnf; k++){  //write the data part
                fprintf(fout, "%s", item1[k]);
            }
            fprintf(fout, "\n");

            if (nf) free_memory_2d(refln, nf);
            if (nnf) free_memory_2d(item1, nnf);
	      
            if (VERB==0) delete_file(tmpf);
            if (VERB==0) delete_file(tmpf_new);
           
//            printf("%s: block=%d nline=%d  nref=%d  nh=%3d\n", outfile, nb, nline, nf,  nh);
        }
        
    }
    fclose(fout);
    if (nline) free_memory_2d(line, nline);
 
}
void cif_dic_check(char *ciffile)
{
    char command[256];
    printf("Checking pdbx mmCIF dictionary by (cif_check)..\n");
    
    sprintf(command,  "~annotator/bin/cif_check.csh  %s >/dev/null", ciffile);
    int fake;
    fake = system(command);
    fake++;
}


void precheck_sf(char *inpfile, char *outfile)
//pre-check items(data with *,powder or fiber diffract, unwanted cif items
    
{
    char str[5120],block_id[256] ;
    int n1=0,  nrem=0, nb=0;
    FILE  *finp=NULL, *fout=NULL;

    if((finp=fopen(inpfile, "r")) == NULL ){
        sprintf(INFO, "Error! can not open the file %s\n",inpfile);
        pinfo(INFO, 0);
        exit(0);
        
    }else{
        while(fgets(str, sizeof str, finp) != NULL){
            if (strchr(str,'_') || strchr(str,'#') || strstr(str, "REMARK")) {
                rid_of_front_space(str);
                if (!strncmp(str,"data_", 5)){
                    strcpy(block_id, str+5);
                    rid_of_front_end_space(block_id);
//                    check_data_block_name(inpfile, block_id, nb);
                    nb++;
                }
            }
            
            if (!strncmp(str,"_refln.", 7)){
                compare_refln(inpfile, str);
                n1=0;
                while(fgets(str, sizeof str, finp) != NULL){
                    if (strchr(str,'_') || strchr(str,'#')) rid_of_front_space(str);
                    
                    if (n1++>60 && (str[0]=='_' || !strncmp(str,"data_", 5) || !strncmp(str,"loop_", 5)) ) {
                        break;
                        
                    }else if (!strncmp(str,"_refln.", 7) ){
                        compare_refln(inpfile, str);
                        
                    }else if (strchr(str, '*')){
                        sprintf(INFO, "Error! File (%s) has * in data (1 listed).(%s).",inpfile,str);
                        pinfo(INFO, 0);
                        break;
                        
                    }else if (strstr_case(str,"_refln.pdbx_fiber_coordinate")){
                        sprintf(INFO, "Note: File (%s) is a fiber diffraction. ", inpfile);
                        pinfo(INFO, 0);
                    }
                }
                
            }else if (!strncmp(str,"REMARK ", 7)){
                if(nrem++ ==3){
                    sprintf(INFO, "Warning: File (%s) has REMARKs. (please check & removed)", inpfile);
                    pinfo(INFO, 0);
                }
                   
            }else if (strstr(str,"_pdbx_powder_data.pd_meas_intensity_total")){
                sprintf(INFO, "Note: File (%s) is a powder diffraction.", inpfile);
                pinfo(INFO, 0);
            }
        }
    }

    check_unwanted_cif_item(inpfile);
    
    if (!nb){
        sprintf(INFO, "Error: File (%s) has no data block name (data_tmp is added).", inpfile);
        pinfo(INFO, 0);
    }

    
    if (strlen(outfile)>0){
        rewind(finp);
        fout=fopen(outfile, "w");

        if (!nb){
            fprintf(fout, "data_tmp\n#\n");
        }
        while(fgets(str, sizeof str, finp) != NULL){
            if (strstr(str,"data_")){
                if(strchr(str, '/')) replace_char(str, '/', '_');
                if(strchr(str, ')')) replace_char(str, ')', '_');
                if(strchr(str, '(')) replace_char(str, '(', '_');
                fprintf(fout, "%s", str);
                continue;
            }
            
            
            fprintf(fout, "%s", str);
        }
        fclose(fout);
    }
        
    fclose(finp);
    
}



void rename_cif_token(int nline, char **line, char *ifile)
//correct the cif tokens for this data_block, if the ciflist is in SF file.
//the left column of ciflist is wrong and the right is correct.
{
    char token[512], tmp[512],str[512], item1[512],  item2[512], **cif_items;
    int  i=0, j, k=0, n1=0, ncif=0;

    static const char *ciflist[] = 
        {
" _refln.A_calc_au                          _refln.A_calc ",
" _refln.B_calc_au                          _refln.B_calc ",
" _refln.A_cal                              _refln.A_calc ",
" _refln.B_cal                              _refln.B_calc ",
" _refln.resolution                         _refln.d_spacing ",
" _refln.Fc_au                              _refln.F_calc_au ",
" _refln.F_sigma                            _refln.F_meas_sigma_au ",
" _refln.F_meas_au_sigma                    _refln.F_meas_sigma_au ",
" _refln.F_squared_meas_sigma               _refln.F_squared_sigma ",
" _refln.I_meas_au                          _refln.intensity_meas ",
" _refln.intensity                          _refln.intensity_meas ",
" _refln.intensity_meas_au                  _refln.intensity_meas ",
" _refln.intensity_sigm                     _refln.intensity_sigma ",
" _refln.I_meas_sigma_au                    _refln.intensity_sigma ",
" _refln.intensity_sigma_au                 _refln.intensity_sigma ",
" _refln.intensity_meas_sigma               _refln.intensity_sigma ",
" _refln.intensity_meas_sigma_au            _refln.intensity_sigma ",
" _refln.dfano                              _refln.pdbx_anom_difference ",
" _refln.DF_anomalous                       _refln.pdbx_anom_difference ",
" _refln.ccp4_phase_anom                    _refln.pdbx_anom_difference ",
" _refln.pdbx_phase_anom                    _refln.pdbx_anom_difference ",
" _refln.ccp4_anomalous_diff                _refln.pdbx_anom_difference ",
" _refln.ccp4_SAD_phase_anom                _refln.pdbx_anom_difference ",
" _refln.pdbx_anomalous_diff                _refln.pdbx_anom_difference ",
" _refln.pdbx_SAD_phase_anom                _refln.pdbx_anom_difference ",
" _refln.pdbx_anomalous_meas_au             _refln.pdbx_anom_difference ",
" _refln.I_ano_meas                         _refln.pdbx_anom_difference ",
" _refln.I_ano_sigma                        _refln.pdbx_anom_difference_sigma ",
" _refln.DF_anomalous_sigma                 _refln.pdbx_anom_difference_sigma ",
" _refln.ccp4_phase_anom_sigma              _refln.pdbx_anom_difference_sigma ",
" _refln.pdbx_phase_anom_sigma              _refln.pdbx_anom_difference_sigma ",
" _refln.ccp4_anomalous_diff_sigma          _refln.pdbx_anom_difference_sigma ",
" _refln.ccp4_SAD_phase_anom_sigma          _refln.pdbx_anom_difference_sigma ",
" _refln.pdbx_SAD_phase_anom_sigma          _refln.pdbx_anom_difference_sigma ",
" _refln.pdbx_anomalous_meas_sigma_au       _refln.pdbx_anom_difference_sigma ",
" _refln.F-_meas_au                         _refln.pdbx_F_minus ",
" _refln.pdbx_F_meas_minus                  _refln.pdbx_F_minus ",
" _refln.ccp4_F_meas_minus_au               _refln.pdbx_F_minus ",
" _refln.pdbx_F_meas_minus_au               _refln.pdbx_F_minus ",
" _refln.ccp4_SAD_F_meas_minus_au           _refln.pdbx_F_minus ",
" _refln.F-_meas_sigma_au                   _refln.pdbx_F_minus_sigma ",
" _refln.pdbx_F_meas_minus_sigma            _refln.pdbx_F_minus_sigma  ",
" _refln.ccp4_F_meas_minus_sigma_au         _refln.pdbx_F_minus_sigma ",
" _refln.pdbx_F_meas_minus_sigma_au         _refln.pdbx_F_minus_sigma ",
" _refln.ccp4_SAD_F_meas_minus_sigma_au     _refln.pdbx_F_minus_sigma ",
" _refln.F+_meas_au                         _refln.pdbx_F_plus ",
" _refln.pdbx_F_meas_plus                   _refln.pdbx_F_plus ",
" _refln.ccp4_F_meas_plus_au                _refln.pdbx_F_plus ",
" _refln.pdbx_F_meas_plus_au                _refln.pdbx_F_plus ",
" _refln.ccp4_SAD_F_meas_plus_au            _refln.pdbx_F_plus ",
" _refln.F+_meas_sigma_au                   _refln.pdbx_F_plus_sigma ",
" _refln.pdbx_F_meas_plus_sigma             _refln.pdbx_F_plus_sigma ",
" _refln.ccp4_F_meas_plus_sigma_au          _refln.pdbx_F_plus_sigma ",
" _refln.pdbx_F_meas_plus_sigma_au          _refln.pdbx_F_plus_sigma ",
" _refln.ccp4_SAD_F_meas_plus_sigma_au      _refln.pdbx_F_plus_sigma ",
" _refln.pdbx_HLA                           _refln.pdbx_HL_A_iso ",
" _refln.ccp4_HL_A_iso                      _refln.pdbx_HL_A_iso ",
" _refln.ccp4_SAD_HL_A_iso                  _refln.pdbx_HL_A_iso ",
" _refln.pdbx_SAD_HL_A_iso                  _refln.pdbx_HL_A_iso ",
" _refln.pdbx_HLB                           _refln.pdbx_HL_B_iso ",
" _refln.ccp4_HL_B_iso                      _refln.pdbx_HL_B_iso ",
" _refln.ccp4_SAD_HL_B_iso                  _refln.pdbx_HL_B_iso ",
" _refln.pdbx_SAD_HL_B_iso                  _refln.pdbx_HL_B_iso ",
" _refln.pdbx_HLC                           _refln.pdbx_HL_C_iso ",
" _refln.ccp4_HL_C_iso                      _refln.pdbx_HL_C_iso ",
" _refln.ccp4_SAD_HL_C_iso                  _refln.pdbx_HL_C_iso ",
" _refln.pdbx_SAD_HL_C_iso                  _refln.pdbx_HL_C_iso ",
" _refln.pdbx_HLD                           _refln.pdbx_HL_D_iso ",
" _refln.ccp4_HL_D_iso                      _refln.pdbx_HL_D_iso ",
" _refln.ccp4_SAD_HL_D_iso                  _refln.pdbx_HL_D_iso ",
" _refln.pdbx_SAD_HL_D_iso                  _refln.pdbx_HL_D_iso ",
" _refln.ccp4_I_minus                       _refln.pdbx_I_minus ",
" _refln.ccp4_I_minus_sigma                 _refln.pdbx_I_minus_sigma ",
" _refln.ccp4_I_plus                        _refln.pdbx_I_plus ",
" _refln.ccp4_I_plus_sigma                  _refln.pdbx_I_plus_sigma ",
" _refln.pahse_calc                         _refln.phase_calc ",
" _refln.F.phase_calc                       _refln.phase_calc ",
" _refln.phase_au                           _refln.phase_meas ",
" _refln.sgx_fmap                           _refln.pdbx_FWT ",
" _refln.pdbx_fom_weighted_fmap             _refln.pdbx_FWT ",
" _refln.statu                              _refln.status ",
" _refln.F_status                           _refln.status ",
" _refln.status_au                          _refln.status ",
" _refln.R_free_flag                        _refln.status ",
" _refln.phenix_R_free_flags                _refln.pdbx_r_free_flag ",
" _refln.observed_status                    _refln.status ",
" _refln.waveLEngth_id                      _refln.wavelength_id ",
" _refln.wavelength_di                      _refln.wavelength_id ",
" _refln.index_l>                           _refln.index_l ",
" _refln.fiber_coordinate                   _refln.pdbx_fiber_coordinate ",
" _refln.fiber_F_meas_au                    _refln.pdbx_fiber_F_meas_au ",
" _refln.fiber_layer                        _refln.pdbx_fiber_layer ",
" _audit.update_recor                      _audit.update_record",
" _cell.enrty_id                           _cell.entry_id ",
" _cell.enry_id                            _cell.entry_id ",
" _cell.entry                              _cell.entry_id",
" _cell_entry.id                           _cell.entry_id ",
" _cell.entry.id                           _cell.entry_id",
" _cell.ndb_unique_axis                    _cell.pdbx_unique_axis",
" _diffrn.detail                           _diffrn.details ",
" _diffrn_radiation.id                     _diffrn_radiation.diffrn_id ",
" _pdbx_powder_refln.d_spacing             _refln.d_spacing  ",
" _pdbx_powder_refln.F_squared_calc        _refln.F_squared_calc  ",
" _pdbx_powder_refln.F_squared_meas        _refln.F_squared_meas  ",
" _pdbx_powder_refln.gsas_i100_meas        _refln.pdbx_gsas_i100_meas ",
" _pdbx_powder_refln.index_h               _refln.index_h  ",
" _pdbx_powder_refln.index_k               _refln.index_k  ",
" _pdbx_powder_refln.index_l               _refln.index_l  ",
" _pdbx_powder_refln.observed_status       _refln.status  ",
" _pdbx_powder_refln.phase_calc            _refln.phase_calc  ",
" _radiation.id                            _diffrn_radiation.diffrn_id ",
" __symmetry.entry_id                     _symmetry.entry_id ",
" _symmetry.int_tables_number             _symmetry.Int_Tables_number ",
" _symmetry.ndb_full_space_group_name_H-M   _symmetry.space_group_name_H-M ",
" _symmetry.space_group_name_h-m           _symmetry.space_group_name_H-M ",

" _refln_index_h                            _refln.index_h ",
" _refln_index_k                            _refln.index_k ",
" _refln_index_l                            _refln.index_l ",
" _refln.F_squared_meas                     _refln.intensity_meas ",
" _refln.F_squared_sigma                    _refln.intensity_sigma ",
" _refln_F_squared_meas                     _refln.F_squared_meas ",
" _refln_F_squared_sigma                    _refln.F_squared_sigma ",
" _refln_F_calc                             _refln.F_calc ",
" _refln_observed_status                    _refln.status ",
" _refln_F_squared_calc                     _refln.F_squared_calc ",
" _refln_phase_calc                         _refln.phase_calc ",
" _refln_d_spacing                          _refln.d_spacing ",
" _gsas_i100_meas                           _refln.pdbx_gsas_i100_meas ",
" _refln.gsas_i100_meas                     _refln.pdbx_gsas_i100_meas ",
" _pd_meas_intensity_total                  _pdbx_powder_data.pd_meas_intensity_total",
" _pd_proc_intensity_total                  _pdbx_powder_data.pd_proc_intensity_total",
" _pd_proc_ls_weight                        _pdbx_powder_data.pd_proc_ls_weight",
" _pd_proc_intensity_bkg_calc               _pdbx_powder_data.pd_proc_intensity_bkg_calc",
" _pd_calc_intensity_total                  _pdbx_powder_data.pd_calc_intensity_total",

"END",
        };

    
    cif_items=set_memory_2d(2000, 900);
    ncif=0;
    for(i=0; i<nline; i++){ //get list of cif items
        strcpy(str, line[i]);
        
        if (strchr(str,'_')) {
	  rid_of_front_space(str);
	}
        if (!strncmp(str, "data_", 5)){
            ncif=get_cifitem_list(i, nline, line, cif_items);
            
            for (j=i; j<nline; j++){
                strcpy(str, line[j]);
		if (strchr(str,'_')){
		  rid_of_front_space(str);
		}

                if (j-i>50 &&  !strncmp(str, "data_", 5) ){
                    i=j-1;
                    ncif=0;
		    std::cout << "Starting next data block\n";
                    break;  //next block
                }

                if (str[0]=='_'){
                    sscanf(str, "%s", item1);

                    if (strchr(item1,'_')){ //possible items
                        
                        rid_of_front_end_space(item1);
                        if(!strncmp(item1,"_refln.", 7) ){
                            compare_refln(ifile, item1);
                        }

                        for(k=0; k<500; k++){ //compare with the list
                            if(!strcmp_case(ciflist[k], "END"))break;
                            sscanf(ciflist[k], "%s %s", item2, token);
                
                            if(!strcmp(item1, item2)){ //wrong token
                                n1=check_refln_list(token, ncif, cif_items);
                                if(n1==0){ //No duplicate (token not in cif_items)
                                    
                                    replace_first_str(line[j], item2, token, tmp);
                                    if(!strchr(tmp, '\n')) strcat (tmp, "\n");
                                    line[j]=new char[strlen(tmp)+1];
                                    strcpy(line[j], tmp);
                                    
                                    if(!strcmp(item2, "_diffrn_radiation.id")) continue;
                                    sprintf(INFO,"Warning: File (%s) has %s replaced by %s.",ifile, item2, token);
                                    pinfo(INFO,0);
                                    
                                }else{
                                    sprintf(INFO,"Warning: File (%s) has %s NOT replaced by %s.",ifile, item2, token);
                                    pinfo(INFO,0);
                                }
                                
                                break;
                            }
                        }
                        
                    }
                }
            }
            
        }
    }
    free_memory_2d(cif_items, 2000);
}

int get_cifitem_list(int start, int nline, char **line, char **cif_items)
//get the list of ciftoken. Stops at start of a second datablock.

{
    char str[5120], item1[512];
    int j,n, ncif=0;


    for (j=start; j<nline; j++){
        strcpy(str, line[j]);
        
        if (strchr(str, '_')) rid_of_front_end_space(str);

        if (!strncmp(str, "data_", 5) && (j-start > 40)){
	  return ncif;
	}
        
        if (str[0]=='_'){
            n=sscanf(str, "%s", item1);
            if (n==1 && item1[0]=='_' && strchr(item1,'.')){ //possible items
                strcpy(cif_items[ncif], item1);
                ncif++;
            }
        }
    }
    return ncif;
}

int check_refln_list(char *token, int nlist, char **refln_list)
//Use the list of _refln. to see if the replaced item is in the list

{
    int i, n=0;
        
    for(i=0; i<nlist; i++){
        if(!strcmp(token, refln_list[i])){
            n=1;
            break;
        }
    }
    return n;
}


void compare_refln(char *inpfile, char *item)
{
    int j, n;
    
    static const char *refln[]={
"_refln.A_calc",
"_refln.A_calc_au",
"_refln.A_meas",
"_refln.A_meas_au",
"_refln.B_calc",
"_refln.B_calc_au",
"_refln.B_meas",
"_refln.B_meas_au",
"_refln.class_code",
"_refln.crystal_id",
"_refln.d_spacing",
"_refln.F_calc",
"_refln.F_calc_au",
"_refln.F_meas",
"_refln.F_meas_au",
"_refln.F_meas_sigma",
"_refln.F_meas_sigma_au",
"_refln.F_squared_calc",
"_refln.F_squared_meas",
"_refln.F_squared_sigma",
"_refln.fom",
"_refln.include_status",
"_refln.index_h",
"_refln.index_k",
"_refln.index_l",
"_refln.intensity_calc",
"_refln.intensity_meas",
"_refln.intensity_sigma",
"_refln.mean_path_length_tbar",
"_refln.pdbx_anom_difference",
"_refln.pdbx_anom_difference_sigma",
"_refln.pdbx_DELFWT",
"_refln.pdbx_DELPHWT",
"_refln.pdbx_diffrn_id",
"_refln.pdbx_F_calc_part_solvent",
"_refln.pdbx_F_calc_with_solvent",
"_refln.pdbx_F_minus",
"_refln.pdbx_F_minus_sigma",
"_refln.pdbx_F_plus",
"_refln.pdbx_F_plus_sigma",
"_refln.pdbx_fiber_coordinate",
"_refln.pdbx_fiber_F_meas_au",
"_refln.pdbx_fiber_layer",
"_refln.pdbx_FWT",
"_refln.pdbx_HL_A_iso",
"_refln.pdbx_HL_B_iso",
"_refln.pdbx_HL_C_iso",
"_refln.pdbx_HL_D_iso",
"_refln.pdbx_I_minus",
"_refln.pdbx_I_minus_sigma",
"_refln.pdbx_I_plus",
"_refln.pdbx_I_plus_sigma",
"_refln.pdbx_phase_calc_part_solvent",
"_refln.pdbx_phase_calc_with_solvent",
"_refln.pdbx_PHWT",
"_refln.phase_calc",
"_refln.phase_meas",
"_refln.refinement_status",
"_refln.scale_group_code",
"_refln.sint_over_lambda",
"_refln.status",
"_refln.pdbx_r_free_flag",
"_refln.symmetry_epsilon",
"_refln.symmetry_multiplicity",
"_refln.wavelength",
"_refln.wavelength_id",
"_refln.pdbx_signal",
"_refln.pdbx_signal_status",
"END"
    };
    n=0;
    rid_of_front_end_space(item);
    
    for(j=0; j<500; j++){
        if(!strcmp_case(refln[j], "END"))break;
        if(!strcmp_case(item, refln[j])){
            n=1;
            break;
        }
    }
    
    if (n==0){
        sprintf(INFO, "Warning: File (%s) has item (%s) not in the pdbx dictionary.", inpfile, item);
        pinfo(INFO, 0);
    }

    if (!strcmp(item,"_refln.phenix_R_free_flags")){
        sprintf(INFO,"Note: _refln.phenix_R_free_flags is renamed as _refln.pdbx_r_free_flag!\n");
        pinfo(INFO,0);
    }

    
}

int remove_ciftable(const char *table, int i, int nline, char **line)
// remove a cif table (must be standard cif)
{
    char tmp[256], tmp1[256];
    int j=0,k=0, end=i;

    for (j=i; j<nline; j++){
        sscanf(line[j], "%s", tmp);
        if (tmp[0]==';'){
            for (k=j+1; k<nline; k++){
                sscanf(line[k], "%s", tmp1);
                if (tmp1[0]==';' ){
                    j=k;
                    end=k-1;
                    break;
                }
            }
        }else  if (j-end>1 && (tmp[0]=='#' || j==nline -1)){
            end=j-1;
            break;
        }
    }
    
    return end;
}


void update_items(char *file,char *newfile, char *pdbid, int nblock)
// the header is updated add/remove/change
  
{
    char  tmp[256], tmp1[256],date[30], correct[512], add[256],change[256];
    char **line, **line1, tmpout[512], comment[90000],removed[256];
    int i,k, nline, nline1, n1=0,n2=0,n3=0, ncell=0, nreflns=0,ndiffr=0;
    int nloop=0, entry=0, nexptl=0,nwave1=0,nwave2=0, nwave=0, ntmp=0;
    FILE *fw=NULL, *fw_tmp=NULL;

    strcpy(date, "");
    strcpy(correct, "");
    strcpy(add, "");
    strcpy(change, "");
    strcpy(removed, "");
    strcpy(comment, "");

    line1=get_lines_from_file(file, &nline1);

    sprintf(tmpout, "%s__tmp", file);
    fw_tmp=fopen(tmpout, "w");
    
    for(k=0; k<nline1 ; k++){
        if(!strncmp(line1[k], "_cell.entry_id", 14)){
            ncell++;
        }else if(!strncmp(line1[k], "_reflns.entry_id", 16)) {
            nreflns++;
        }else if (line1[k][0]=='#') {
            guess_comment(line1[k], comment);
        }else if(k<nline1-1 && !strncmp(line1[k],"loop_", 5) &&
                 !strncmp(line1[k+1],"_reflns.", 8) ){
            nloop=1;
        }
           
        if(!strncmp(line1[k],"loop_", 5) &&
           strstr(line1[k+1],"_symmetry_equiv.") &&
           strstr(line1[k+2],"_symmetry_equiv.") &&
           !strstr(line1[k+3],"_symmetry_equiv.")&&
           (strchr(line1[k+3],'_') ||strchr(line1[k+3],'#'))){
            k=k+2;
            sprintf(INFO,"Warning: File (%s) empty table _symmetry_equiv (removed).", pdbid);
            pinfo(INFO,0);
            
            strcat (removed, "_symmetry_equiv, ");
            continue;
            
        }else{
            
            if (!strncmp(line1[k], "_reflns.entry_id",16)){
                if (sscanf(line1[k], "%s %s" , tmp1,tmp)==2){
                    continue;
                }
            }
            
            fprintf(fw_tmp, "%s", line1[k]);
        }
        
    }
    fclose(fw_tmp);

    
    if (ncell>1) {
        sprintf(INFO, "Warning: (%s) _cell appeared twice(manual check).", pdbid);
        pinfo(INFO,0);
    }
    if (nreflns>1) {
        sprintf(INFO, "Warning: (%s) _reflns appeared twice(manual check).", pdbid);
        pinfo(INFO,0);
    }
    
    if (strlen(comment)>10){
        sprintf(INFO, "Warning: (%s) Text below is put into _diffrn.details. (Check carefully)!", pdbid);
        pinfo(INFO,0);
        
        printf("Text in %s: (%s)\n", pdbid, comment);
        strcat(add, "_diffrn to hold comments, ");
    }
    
    if (nloop==1) {
        sprintf(INFO, "Note: (%s) _reflns is looped.", pdbid);
        pinfo(INFO,0);
    }
    
    
    cifparse(tmpout, tmpout, 200) ;   //make output be standard cif format    
    line=get_lines_from_file(tmpout, &nline);    
    for(k=0; k<nline ; k++){
        if(!strncmp(line[k], "_diffrn.", 8)) {
            ndiffr++;
        }else if(!strncmp(line[k], "_diffrn_radiation.pdbx_wavelength", 33)){
            ntmp = sscanf(line[k], "%s %s" , tmp1,tmp);
            if (ntmp==2 && (strstr(line[k], "0.000") ||strstr(line[k], "?")|| strstr_case(line[k], "nul")) ){
                nwave1=1;
            }
            
            nwave2=1;
            
        }else if(!strncmp(line[k], "_diffrn_radiation_wavelength.wavelength", 39)){
            ntmp = sscanf(line[k], "%s %s" , tmp1,tmp);
            if (ntmp==2 && (strstr(line[k], "0.000") ||strstr(line[k], "?")|| strstr_case(line[k], "nul")) ){
                nwave=1;
            }
            nwave2=1;
            
        }else if(!strncmp(line[k], "_entry.id", 9)){
            entry++;
        }else if(!strncmp(line[k], "_exptl_crystal.id", 17)){
            nexptl++;
        }
    }
        /*
    if(nwave1 || nwave){
        sprintf(INFO, "Warning: SFfile (%s) has wrong value of wavelength (auto removed!).", pdbid);
        pinfo(INFO,0);
    }
        */        

//    printf("numbers= %d %d %d %d %d\n", ndiffr, nwave1, nwave, entry, nexptl);
    
    fw=fopen(newfile, "w");
    
    for(i=0; i<nline; i++){ //remove items (then add _audit &_diffrn back)
        if (i<nline-1 && (!strncmp(line[i],"loop_", 5) || line[i][0]=='#') &&
            (!strncmp(line[i+1],"_database_2.", 12) || !strncmp(line[i+1],"_atom_sites.", 12)||
             !strncmp(line[i+1],"_audit.", 7) || !strncmp(line[i+1],"_diffrn.", 8))){
            
            if (!strncmp(line[i+1],"_database_2.", 12) ){
                i=remove_ciftable("_database_2.", i, nline, line);

            }else if (!strncmp(line[i+1],"_atom_sites.", 12) ){
                i=remove_ciftable("_atom_sites.", i, nline, line);

            }else if (!strncmp(line[i+1],"_audit.", 7) ){
                i=remove_ciftable("_audit.", i, nline, line);
                
            }else if (!strncmp(line[i+1],"_diffrn.", 8) ){
                i=remove_ciftable("_diffrn.", i, nline, line);
            }

            
        }else if (i<nline-1 && nwave1 && (!strncmp(line[i],"loop_", 5) || line[i][0]=='#') &&
                  !strncmp(line[i+1],"_diffrn_radiation.", 18)){
            
            i=remove_ciftable("_diffrn_radiation.", i, nline, line);
            strcat(removed, "_diffrn_radiation, ");
            
        }else if (i<nline-1 && nwave && (!strncmp(line[i],"loop_", 5) || line[i][0]=='#') &&
                  !strncmp(line[i+1],"_diffrn_radiation_wavelength.", 29)){
            
            i=remove_ciftable("_diffrn_radiation_wavelength.", i, nline, line);
            strcat(removed, "_diffrn_radiation_wavelength, ");
           
            
        }else if(!strncmp(line[i],"_diffrn_radiation.", 18) ){
            replace_first_str(line[i], "_diffrn_radiation.", "_diffrn_radiation_wavelength.", tmp);
            if (strstr(tmp,"wavelength.diffrn_id")){
                replace_first_str(tmp, "wavelength.diffrn_id", "wavelength.id", tmp);
            }else if (strstr(tmp,".pdbx_wavelength")){
                replace_first_str(tmp, ".pdbx_wavelength", ".wavelength", tmp);
                strcat(change, "_diffrn_radiation to _diffrn_radiation_wavelength, ");
            }
            
            fprintf(fw, "%s",  tmp);
            
        }else if(!strncmp(line[i],"_exptl.crystal_F_000", 20)){
            
            strcat(change, "_exptl.crystal_F_000 to _exptl_crystal.F_000, ");
            replace_first_str(line[i], "_exptl.crystal_F_000", "_exptl_crystal.F_000", tmp);
            fprintf(fw, "%s",  tmp);
            continue;
            
            
        }else if(!strncmp(line[i],"_cell.", 6)){
            n1++;
            if (sscanf(line[i], "%s %s", tmp,  tmp1)==2){
                if (strstr(tmp,"_cell.formula_units_Z") &&  atoi(tmp1)==0 ) {
                    strcat(removed, "_cell.formula_units_Z, ");
                    continue;
                }
            }
            
            if (n1==1 && !strstr_case(line[i],"entry_id")){
                fprintf(fw, "_cell.entry_id    %s \n", pdbid);
                fprintf(fw, "%s", line[i]);
                strcat(add, "_cell.entry_id, ");
            }else{
                fprintf(fw, "%s", line[i]);
            }
            
        }else if(!strncmp(line[i],"_symmetry.", 10)){
            n2++;
            if (n2==1 && !strstr_case(line[i],"entry_id")){
                fprintf(fw, "_symmetry.entry_id    %s \n", pdbid);
                fprintf(fw, "%s", line[i]);
                strcat(add, "_symmetry.entry_id, ");
            }else{
                fprintf(fw, "%s", line[i]);
            }
            
        }else if(!strncmp(line[i],"_audit_author.ordinal", 21)){
            replace_first_str(line[i], ".ordinal", ".pdbx_ordinal", tmp);
            fprintf(fw, "%s", tmp);
            strcat(change, "_audit_author.ordinal to _audit_author.pdbx_ordinal, ");
            
        }else if(!strncmp(line[i],"_refln.", 7)){
            if (sscanf(line[i], "%s %s", tmp, tmp1)==2){
                strcpy(tmp, line[i]);
                delete_return(tmp);
                sprintf(INFO, "Warning: pdbid (%s) has (%s) auto-removed", pdbid, tmp);
                pinfo(INFO,0);
                strcat(removed, tmp);
                continue; 
            }
            
            
        }else if(!strncmp(line[i],"_reflns.", 8)){
            n3++;
            if (n3==1) {
                strcat(change, "_reflns to _diffrn_reflns, ");
                if (nloop==1){
                    fprintf(fw, "_diffrn_reflns.diffrn_id   \n");
                }else{
                    fprintf(fw, "_diffrn_reflns.diffrn_id   1 \n");
                }
                    
                strcat(add, "_diffrn_reflns.diffrn_id, ");
            }
            if (!strncmp(line[i],"_reflns.entry_id", 16))continue;
            
            strcpy(tmp,strchr(line[i],'.')+1);
            
            if(strstr(tmp, "d_resolution_high")){
                replace_first_str(tmp, "d_resolution_high", "pdbx_d_res_high", tmp);
            }else if(strstr(tmp, "d_resolution_low")){
                replace_first_str(tmp, "d_resolution_low", "pdbx_d_res_low", tmp);
            }else if (strstr(tmp, "number_all")){
                replace_first_str(tmp, "number_all", "number", tmp);
            }else if (strstr(tmp, "number_obs")){
                replace_first_str(tmp, "number_obs", "pdbx_number_obs", tmp);
            }
            
            fprintf(fw, "_diffrn_reflns.%s", tmp);
            
        }else{
            fprintf(fw, "%s", line[i]);
            
        }
    }

    if (nwave2==0 && nblock>1) {//add wavelength only to satisfy the cif check
        fprintf(fw, "\n#\n_diffrn_radiation_wavelength.id       1  \n");
        fprintf(fw, "_diffrn_radiation_wavelength.wavelength    .\n#\n");
    }
    
    
    if (ndiffr==0 && n3>0 && !strlen(comment)){
        fprintf(fw, "#\n_diffrn.crystal_id   1 \n");
        fprintf(fw, "_diffrn.id   1 \n#\n");
        strcat(add, "_diffrn.id, _diffrn.crystal_id, ");
    }

    if (entry==0){
        fprintf(fw, "#\n_entry.id   %s \n", pdbid);
        strcat(add, "entry.id, ");
    }
    
    if (nexptl==0){
        fprintf(fw, "#\n_exptl_crystal.id   1 \n");
        strcat(add, "_exptl_crystal.id, ");
    }
    
    
    if (strlen(add)>1) {
        strcat (correct, "ADDED: ");
        strcat (correct, add);
        strcat (correct, "\n");
    }
    
    if (strlen(removed)>1){
        strcat (correct, "REMOVED: ");
        strcat (correct, removed);
        strcat (correct, "\n");
    }
    
    if (strlen(change)>1) {
        strcat (correct, "CHANGED: ");
        strcat (correct, change);
        strcat (correct, "\n");
    }
    
//    if(strlen(correct)>0)printf("Warning: CIF items corrected(pdbid=%s)\n%s",pdbid,correct);
    if (strlen(correct)<5) strcpy(correct, " reformated header part ");

    
    parse_and_update(fw, tmpout,  date,  correct, comment, nblock);
    
    if (VERB==0) delete_file(tmpout);
    
    fclose(fw);
    free_lines_from_file(line1, nline1);
    free_lines_from_file(line, nline);    
    
    
}

int test_comment(char *str)
{
    char tmp[9000];
    int n=1;

    sscanf(str,"%s", tmp);

    if(tmp[0]=='_'){
        if (strchr(tmp, '.')|| strchr(tmp+1, '_')) n=0;
    }else{
        if (!strncmp(tmp, "loop_", 5) || !strncmp(tmp, "data_", 5)) n=0;
        if(strstr(str, "../")|| strstr(str, "; Created by mtz2various (CCP4) from")) n=0;
    }
    
    return n;
}

void guess_comment(char *line, char *comment)
{
    char str[512];
    int c, i, len=0, nlett=0;

    strcpy(str, line);
    
    len=strlen(str);

    for (i=0; i<len; i++){
        c=str[i];
        
        if (c>=97 && c<=122){ nlett++;}
    }

    replace_char(str,'#', ' ');
    
    if (strlen(comment)<2 && nlett>10){ //possible comments (1st)
        if(!test_comment(str)) return;
        strcat (comment,str);
        
    }else if(strlen(comment)>9 && nlett>5){ //possible comments
        if(!test_comment(str)) return;
        strcat (comment,str);
        
    }
    rid_of_front_space(comment);

}

void get_cifobj(char *infile, CifFile *fobj,const char *Bid)
{
    string diags, blockId;
    
    CifFile * fobjR = NULL;
    CifParser* cifParserR = NULL;
    fobjR = new CifFile();
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(infile, diags);
    delete(cifParserR);
    

    blockId = fobjR->GetFirstBlockName();
    Bid=blockId.c_str();
}


void parse_and_update(FILE *fout, char *infile,  char *date,  char *correct,
                      char *comment, int nblock)
{
    
    char **aud_date=NULL,**aud_meth=NULL,**aud_rec=NULL, **aud_id=NULL;
    char **dif_id=NULL,**dif_cid=NULL,**dif_detail=NULL, **temp=NULL, **treat=NULL;
    const char *Bid;
    long  naud=0, ndif=0;
    string diags, blockId;

    CifFile * fobjR = NULL;
    CifParser* cifParserR = NULL;
    fobjR = new CifFile();
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(infile, diags);
    delete(cifParserR);

    blockId = fobjR->GetFirstBlockName();
    Bid=blockId.c_str();
//    get_cifobj(infile, fobjR, Bid);
    
    aud_id=get_attribute_values(fobjR, Bid,"audit", "revision_id", &naud);
    aud_rec=get_attribute_values(fobjR, Bid,"audit", "update_record", &naud);
    aud_meth=get_attribute_values(fobjR, Bid,"audit", "creation_method", &naud);
    aud_date=get_attribute_values(fobjR, Bid,"audit", "creation_date", &naud);

    if(nblock==1)write_audit(fout, date, correct, naud, aud_id, aud_rec, aud_date,aud_meth, 1);

    
    dif_id=get_attribute_values(fobjR, Bid,"diffrn", "id", &ndif);
    dif_cid=get_attribute_values(fobjR, Bid,"diffrn", "crystal_id", &ndif);
    temp=get_attribute_values(fobjR, Bid,"diffrn", "ambient_temp", &ndif);
    treat=get_attribute_values(fobjR, Bid,"diffrn", "crystal_treatment", &ndif);
    dif_detail=get_attribute_values(fobjR, Bid,"diffrn", "details", &ndif);
    
//    if((dif_cid || dif_detail || temp || treat || strlen(comment)>1) && strlen(DETAIL)==0 ){
    if((dif_cid || dif_detail || temp || treat || strlen(comment)>1)){
        write_diffrn(fout, ndif, dif_id, dif_cid, dif_detail, temp, treat, comment);
    }

    delete_attribute_values(aud_id);
    delete_attribute_values(aud_rec);
    delete_attribute_values(aud_meth);
    delete_attribute_values(aud_date);    
    delete_attribute_values(dif_id);
    delete_attribute_values(dif_cid);
    delete_attribute_values(temp);
    delete_attribute_values(dif_detail);    
    if (fobjR){
        delete(fobjR);
    }
    
}

void write_diffrn(FILE *fout, long ndif, char **dif_id,char **dif_cid, 
                  char **dif_detail,char **temp, char **treat, char *comment)
{
    char tmp1[90000], tmp2[90000];
    int i=0, k=1;
    

    strcpy(tmp1, "");
    strcpy(tmp2, "");
    
    fprintf(fout,"#\nloop_\n");
    fprintf(fout,"_diffrn.id \n");
    fprintf(fout,"_diffrn.crystal_id \n");
    fprintf(fout,"_diffrn.ambient_temp \n");
    fprintf(fout,"_diffrn.crystal_treatment \n");
    fprintf(fout,"_diffrn.details \n");
        
    if (strlen(DETAIL)>0) { //
        fprintf(fout,"1 1 ? ? \n;%s\n;\n",  DETAIL);
        return;
    }
        
    
    for(i=0; i<ndif; i++){
        if(dif_id) {
            fprintf(fout,"'%s' ", dif_id[i]);
        }else{
            (i==0) ? fprintf(fout," 1 ") : fprintf(fout," ? ");
        }
            
        if(dif_cid) {
            fprintf(fout,"'%s' ", dif_cid[i]);
        }else{
            (i==0) ? fprintf(fout," 1 ") : fprintf(fout," ? ");
        }
            
        (temp) ? fprintf(fout,"'%s' ", temp[i]) : fprintf(fout," ? ");
        (treat) ? fprintf(fout," \"%s\" ", treat[i]) : fprintf(fout," ? ");

        strcpy(tmp1,"");
        if(dif_detail) {
//            printf("helll= %d  |%s| : %s ",i , DETAIL, dif_detail[i]);
            strcpy(tmp1, dif_detail[i]);
//            if ( strlen(DETAIL)>0 && i==0)  strcat (tmp1, DETAIL);
            
            replace_char(tmp1, '#', ' ');
            fprintf(fout," \n;%s\n;", tmp1) ;
            replace_char(tmp1, '\n', ' '); //for next comparision
            rid_of_front_end_space(tmp1);
            
        }else{
            fprintf(fout," ? ");
        }
        
        fprintf(fout,"\n");
        
    }
    k=ndif+1;
    
    if (strlen(comment)>1){
        strcpy(tmp2, comment);
        replace_char(tmp2, '#', ' ');
        replace_char(tmp2, '\n', ' ');
        rid_of_front_end_space(tmp2);
    }
    
    if (strlen(comment)>1 && !strstr_space(tmp1, tmp2) ){
        fprintf(fout,"%d 1 ? ? \n;%s\n;\n", k, comment);
    }

        
    

}


void write_audit(FILE *fout, char *date, char *correct,long naud,char **aud_id,
                 char **aud_rec,char **aud_date, char **aud_meth, int id)
{
    char date_old[256], tmp1[256], tmp2[256];
    int j, n=0, nt=0;


    strcpy(date_old, "?"); //the original date
      
    
    fprintf(fout,"#\nloop_\n");
    fprintf(fout,"_audit.revision_id  \n");
    fprintf(fout,"_audit.creation_date  \n");
    fprintf(fout,"_audit.update_record  \n");
   // if(naud && aud_meth) fprintf(fout,"_audit.creation_method  \n");


    
    if (strlen(AUDIT)>0){
        nt=sscanf(AUDIT, "%s %s", tmp1, tmp2);
        n=strlen(tmp1);
        
        if (!(n==10 && !strncmp(tmp1, "20", 2))){
            printf("Error: wrong date (%s) format.(it should be yyyy-mm-dd)\n", tmp1);
        }
        if (nt==2){ //user added contents
            sscanf(AUDIT, "%s", date);
            strcpy(correct, get_substr(AUDIT,":", "@"));
        }else{
            strcpy(date, tmp1);
        }
        
    }

    if (!naud){
        fprintf(fout," 1_0  ?  'Initial release'  \n");
        
        if (strlen(AUDIT)>1){
            fprintf(fout,"1_1  %s  \n;%s\n; \n",  date, correct);
        }
        return;
    }

    
    n=0;
       
    for(j=0; j<naud; j++){

        if (aud_id) {
            fprintf(fout,"'%s' ", aud_id[j]);
        }else{
            fprintf(fout," 1_%d  ", j);
        }

        if(aud_date){
            if (j==0 && strchr(aud_date[j],'?')){
                fprintf(fout," %s ", date_old );
            }else{
                fprintf(fout," %s ", aud_date[j]);
            }
        }else{
            fprintf(fout," ? ");            
        }


        if(j==0 && !aud_rec){
            fprintf(fout," 'Initial release' \n"); 
        }else{
            (aud_rec) ? fprintf(fout,"\n;%s\n;\n ", aud_rec[j]) : fprintf(fout," ? \n");
        }

        n++;
    }
        
    if (strlen(AUDIT)>1 and id>0){ //add audit
        fprintf(fout,"1_%d  %s   \n;%s\n; \n", n,  date, correct);
    }
        
}    


void correct_list(char *inpf1)
/* correct a list of files (3 column in each row)
   pdbid, date(as 2012-02-26), "corrections"
 */
{
    char  infile[256], ofile[256], id1[20], correct[512], command[512];
    char **line1,  date[20],date_new[30], pdbfile[256];
    int  i, nline1;
    
    line1=get_lines_from_file(inpf1, &nline1);
    
    printf("Doing sf correction for the list (%s)\n", inpf1);
    
    for(i=0; i<nline1; i++){
        strcpy(date,"?");
        
        sscanf(line1[i],"%s%s", id1, date_new);
        strcpy(correct, get_substr(line1[i],"\"","\""));
        rid_of_front_end_space(correct);
               
        if(strlen(id1) != 4 ) continue;
        
        sprintf(infile, "r%ssf.ent.mmcif", id1);//the corrected file
        sprintf(ofile, "r%ssf.ent_new", id1);
        
        sprintf(command, "/bin/csh -c \" auto -pdb  %s \" ",id1);
        int fake;
        fake = system(command);
        fake++;
        sprintf(pdbfile, "pdb%s.ent", id1);
        
//        make_sf_standard(infile, pdbfile, ofile, correct, '?', date_new, id1);
    
        printf("\nThe new file is %s (%s  %s %s)\n", ofile, id1,date_new, correct );
        
//        sprintf(command, "/bin/csh -c \" rm -f *_tmp? *_temp?; echo ===correcting %s  ===; diff  %s  %s \" ",infile,   infile, ofile);
//        system(command);
    }
    if(nline1 && line1) free_memory_2d(line1, nline1);
    

}


void correct_list_file(char *inpf1, char *inpf2)
{
    char  infile[256], ofile[256], id1[20], id2[20], correct[256], command[512];
    char **line1, **line2, date[20],date_new[30], pdbfile[256];
    int  i,j, nline1, nline2;

    strcpy(date_new, "2010-02-16"); //the new date for release

    
    line1=get_lines_from_file(inpf1, &nline1);
    line2=get_lines_from_file(inpf2, &nline2);
    
    printf("Doing sf correction (%d %d)\n", nline1,nline2);
    for(i=0; i<nline1; i++){
        strcpy(date,"?");
        
        sscanf(line1[i],"%s", id1);
        if(strlen(id1)<4 || id1[0] =='!' ) continue;
        
//        sprintf(infile, "r%ssf.ent_new", id1);
        sprintf(infile, "r%ssf.ent.mmcif", id1);//the corrected file
        sprintf(ofile, "r%ssf.ent_new", id1);
        
        sprintf(command, "/bin/csh -c \" auto -pdb  %s \" ",id1);
        int fake;
        fake = system(command);
        fake++;
        sprintf(pdbfile, "pdb%s.ent", id1);
        
        strcpy(correct,line1[i]+5);
        rid_of_front_end_space(correct);

        for(j=0; j<nline2; j++){ //look for date of the ID
            sscanf(line2[j],"%s", id2);
            if(!strcmp_case(id1, id2)){
                sscanf(line2[j],"%*s%*s%s", date);
                break;
            }
        }
        
        make_sf_standard(infile, pdbfile, ofile, correct, date,date_new, id1);
        
    
        printf("\nThe new file is %s\n", ofile);
        
//        sprintf(command, "/bin/csh -c \" rm -f *_tmp? *_temp?; echo ===correcting %s  ===; diff  %s  %s \" ",infile,   infile, ofile);
//        system(command);
    }
    if(nline1 && line1) free_memory_2d(line1, nline1);
    if(nline2 && line2) free_memory_2d(line2, nline2);
    

}



void update_audit_new(char *infile,  char *date, char *date_new, char *correction)
{
    char  **line, **aud_date=NULL,**aud_rec=NULL;
    int id=0, nline, i, j;
    long  naud=0;
    string diags;

    FILE *fout;
    


//    sprintf(outf, "%s_audit", infile);
    printf("Updating audit record(%s)...\n", infile);
    
    
    cifparse(infile, infile, 100) ;
    line=get_lines_from_file(infile, &nline);
    
    
    for(i=0; i<nline; i++){
        rid_of_front_space(line[i]);
        if(!strncmp_case(line[i], "_audit.", 7)) id=1;
    }

    
    if(id>0){
        CifFile * fobjR = NULL;
        CifParser* cifParserR = NULL;
        fobjR = new CifFile();
        cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
        cifParserR->Parse(infile, diags);
        delete(cifParserR);
        const char *Bid;
        string blockId;

        blockId = fobjR->GetFirstBlockName();
    
        Bid=blockId.c_str();
//        aud_id=get_attribute_values(fobjR, Bid,"audit", "revision_id", &naud);
        aud_date=get_attribute_values(fobjR, Bid,"audit", "creation_date", &naud);
        aud_rec=get_attribute_values(fobjR, Bid,"audit", "update_record", &naud);
        
    }
    fout=fopen(infile, "w");
    
    for(i=0; i<nline; i++){
        if(!strncmp(line[i],"data_", 5)){
            fprintf(fout,"%s", line[i]);
            
            fprintf(fout,"#\nloop_\n");
            fprintf(fout,"_audit.revision_id  \n");
            fprintf(fout,"_audit.creation_date  \n");
            fprintf(fout,"_audit.update_record  \n");

            if(aud_date) {
                for(j=0; j<naud; j++){
                    fprintf(fout,"1_%d  ", j);
                    
                    if (j==0 && aud_date[j][0]=='?'){
                        fprintf(fout," %s ", date );
                    }else{
                        fprintf(fout," %s ", aud_date[j]);
                    }
                    
                    if(j==0 && !aud_rec){
                        fprintf(fout," 'Initial release' \n"); 
                    }else{
                        (aud_rec) ? fprintf(fout," '%s' \n", aud_rec[j]) : fprintf(fout," ? \n");
                    }
                }
                fprintf(fout,"1_%d  %s  '%s' \n", j, date_new, correction);
                
            }else{
                fprintf(fout,"1_0  %s  'Initial release' \n",  date);
                fprintf(fout,"1_1  %s  '%s' \n", date_new, correction);
            }
            
        }else if((strstr(line[i],"loop_") && strstr(line[i+1],"_audit.")) ||
                 strstr(line[i],"_audit.")){//skip audit
            for(j=i; j<nline; j++){
                rid_of_front_space(line[j]);
                if (line[j][0]=='#'){
                    i=j;
                    break;
                }
                continue;
            }
        }else{
            fprintf(fout,"%s", line[i]);
        }
    }
    fclose(fout);

}


void make_sf_standard(char *sffile, char *pdbfile ,char *ofile ,
                      char *correct, char *date, char *date_new,char* pdb_id)
{
    char outf_tmp1[256], outf_tmp2[256], outf_tmp3[256] , **line;
    int i,j, nline, no=0, nf=0, ns=0;
    FILE  *fw,  *fw1, *fw2;
   
    sprintf(outf_tmp1, "%s_temp1", sffile);
    sprintf(outf_tmp2, "%s_temp2", sffile);
    sprintf(outf_tmp3, "%s_temp3", sffile);

    fw=fopen(ofile, "w");
    fw1=fopen(outf_tmp1, "w");
    fw2=fopen(outf_tmp2, "w");
        
    line=get_lines_from_file(sffile, &nline); //seperate 1st and 2st parts
    for(i=0; i<nline; i++){
        if(strstr(line[i],"loop_") && strstr(line[i+1],"_refln.")){
            fprintf(fw2,"data_r%ssf\n", pdb_id);
            for(j=i; j<nline; j++){
                fprintf(fw2,"%s",line[j]);
            }
            i=j;
            
        }else{
            fprintf(fw1,"%s",line[i]);
        }
        
    }
    fclose(fw1);
    fclose(fw2);
    if(line && nline) free_memory_2d(line,nline);
     
    cifparse(outf_tmp1, outf_tmp1, 100) ;
    make_complete_SF(pdbfile, outf_tmp1);
    update_audit_new(outf_tmp1, date, date_new, correct);

    
    line=get_lines_from_file(outf_tmp1, &nline);
    for(i=0; i<nline; i++){
        fprintf(fw, "%s", line[i]);
    }
    if(line && nline) free_memory_2d(line,nline);


    line=get_lines_from_file(outf_tmp2, &nline);
    for(i=0; i<nline; i++){
        if(strstr(line[i], " o")){
            no++;
        }else if(strstr(line[i], " f")){
            nf++;
        }else if (strstr(line[i], "_refln.status")){
            ns=1;
        }else if(i>100 && strstr(line[i], "data_")){
            break;
        }
    }
    if(line && nline) free_memory_2d(line,nline);
    

    if(ns==1 && (no<200 || (nf>0 && no<nf))){ //status exist
        cif2cif_sf_all(outf_tmp2, outf_tmp3, pdb_id, 1);
        line=get_lines_from_file(outf_tmp3, &nline);
    }else{
        cifparse(outf_tmp2, outf_tmp2, 100) ;
        line=get_lines_from_file(outf_tmp2, &nline);
    }
    

    for(i=0; i<nline; i++){
        if(i<10 && strstr(line[i], "data_")) continue;
        fprintf(fw, "%s", line[i]);
    }
    fprintf(fw, "#END OF REFLECTIONS\n");
    if(line && nline) free_memory_2d(line,nline);
    fclose(fw);
}
 

/*            
void insert_column(char *infile, const char *token)
{
    char  **line;
    int i,j,id=0, nline=0;
    FILE *fw=NULL;
    

    line=get_lines_from_file(infile, &nline);

    for(i=0; i<nline; i++){
        if (strstr_case(line[i], token)) {
            id=1;
            printf("Warning! The %s is not in %s and will be inserted as 1\n", token,infile );
            break;
        }
        if(i>400) break;
    }

    if(id <=0) return;

    fw=fopen(infile, "w");
    for(i=0; i<nline; i++){
        fprintf(fw,"%s\n", line[i]);
        
        if(strstr_case(line[i], "loop_") && strstr_case(line[i+1], "_refln.") ){
            fprintf(fw,"%s\n", token);
            fprintf(fw,"%s", line[i+1]);
            for(j=i+2; j<nline; j++){
                if(strchr(line[j], '#')) break;
                if( !strstr_case(line[j], "_refln.")){
                    fprintf(fw, " 1 %s", line[j]);
                }else{
                    fprintf(fw, "%s", line[j]);
                }
            }
        }
    }
    fclose(fw);
    
    if(nline) free_memory_2d(line,nline);
    

}
*/


void add_audit(char *outfile)
{

    char **line;
    int i, nline=0, n=0;
    FILE *fw=NULL;
    
    line=get_lines_from_file(outfile, &nline);
    
    
    for(i=0; i<nline; i++){
        if(strstr_case(line[i], "_audit.")){
            n=1;
            break;
        }
    }

    if(n==0){
        fw=fopen(outfile, "w");
        for(i=0; i<nline; i++){
            if(strstr(line[i],"data_")){
                fprintf(fw,"%s",line[i]);
                audit_sf_header(fw,0);
            }else{
                fprintf(fw,"%s",line[i]);
            }
        }
        fclose(fw);
    }
    if(nline)free_memory_2d(line,nline);
    
}


void update_audit(char *inpf)
// given a list containing pdbid and corrections
{
    char **line, infile[256], ofile[256], id[20], correct[256], command[1024];
    int nline=0, i;

    line=get_lines_from_file(inpf, &nline);
    
    for(i=0; i<nline; i++){
        
        strncpy(id,line[i], 4);
        id[4]='\0';
        rid_of_space(id);
        if(strlen(id)<4 || id[0] =='!' ) continue;
        
        sprintf(infile, "r%ssf.ent_new", id);
        sprintf(ofile, "r%ssf.ent_new_audit", id);
        strcpy(correct,line[i]+5);
        rid_of_front_end_space(correct);
        
        add_items_to_cif(infile, ofile, correct);
        
//        reformat_file(ofile, ofile);
    
        printf("\nThe new file is %s\n", ofile);
        
        sprintf(command, "/bin/csh -c \" rm -f *_tmp?; echo ===correcting %s  ===; diff  %s  %s \" ",infile,   infile, ofile);
        int fake;
        fake = system(command);
        fake++;
    }

}

void add_items_to_cif(char *inpfile, char *outfile, char *correct)
{
    char  **line, str[512], ftmp1[512], ftmp2[512], cate[80], item[80];
    char **aud_date=NULL,**aud_rec=NULL, date[80] ;
    long  naud=0, na=0;
    int nline, i,j=0,key=0;
    string diags;
    FILE *fout=NULL, *fo1=NULL, *fo2=NULL;


    strcpy(date, "2009-09-01");

//    sprintf(inpfile, "%s_inp_tmp1", inpfile_inp);, inpfile[512]
//    reformat_file(inpfile_inp, inpfile);
    
    line=get_lines_from_file(inpfile, &nline);
    
    sprintf(ftmp1, "%s_tmp1", inpfile);
    sprintf(ftmp2, "%s_tmp2", inpfile);
    
    fout=fopen(outfile,"w");
    fo1=fopen(ftmp1,"w");
    fo2=fopen(ftmp2,"w");

    
    
    for(i=0; i<nline; i++){

        if((strstr(line[i],"loop_") && strstr(line[i+1],"_audit.") && i<nline-2 && key==0) ||
           (strstr(line[i],"_audit.") && key==0) ){
            fprintf(fo1,"data_tmp\n");
            
            for(j=i; j<nline; j++){
                rid_of_front_space(line[j]);
                if(line[j][0]=='#' || (!strncmp(line[j],"loop_", 5)&& j>i) ||
                   (check_category(line[j],cate, item) && !strstr(cate, "audit") )) {
                    fprintf(fo2,"%s", line[j]);
                    i=j+1;
                    
                    break;
                    
                }
                fprintf(fo1,"%s", line[j]);
                na++;
                
            }
            key=1;
            
        }
        
        fprintf(fo2,"%s", line[i]);
    }
    fclose(fo1); fclose(fo2);
    
    if(nline) free_memory_2d(line, nline);
    
    if(na>0){
        CifFile * fobjR = NULL;
        CifParser* cifParserR = NULL;
        fobjR = new CifFile();
        cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
        cifParserR->Parse(ftmp1, diags);
        delete(cifParserR);
        const char *Bid;
        string blockId;

        blockId = fobjR->GetFirstBlockName();
    
        Bid=blockId.c_str();
//        aud_id=get_attribute_values(fobjR, Bid,"audit", "revision_id", &naud);
        aud_date=get_attribute_values(fobjR, Bid,"audit", "creation_date", &naud);
        aud_rec=get_attribute_values(fobjR, Bid,"audit", "update_record", &naud);
    }
    
    nline=0;
    
    line=get_lines_from_file(ftmp2, &nline);
//    printf("the new file is %s:: %d\n", ftmp2,nline );

    
    for(i=0; i<nline; i++){
        strcpy(str, line[i]);
        rid_of_front_space(str);

        if(!strncmp(str,"data_", 5) && i<200){
            fprintf(fout,"%s", line[i]);
            
//            if(na>0){
                fprintf(fout,"#\nloop_\n");
                fprintf(fout,"_audit.revision_id  \n");
                fprintf(fout,"_audit.creation_date  \n");
                fprintf(fout,"_audit.update_record  \n");
                for(j=0; j<naud; j++){
                     fprintf(fout,"1_%d  ", j);
                    if(aud_date) {
                        rid_of_space(aud_date[j]);
                        fprintf(fout,"%s  ", aud_date[j]) ;
                    }else
                        fprintf(fout," ? ");
                     
                    (aud_rec) ? fprintf(fout,"'%s' ", aud_rec[j]) : fprintf(fout," ? ");
                    fprintf(fout,"\n");
                }
                fprintf(fout,"1_%d  %s  '%s' \n", j, date, correct);
//            }
            
        }else {
            fprintf(fout,"%s", line[i]);
        }
    }
    fclose(fout);
    if(nline)free_memory_2d(line, nline);
    
    
}


void reformat_file(char *inpfile, char *outfile )
// only reformat the header section
{
    char  **line, ftmp1[512], ftmp2[512],command[2048];
    int nline, i, j, n1=0;
    FILE  *fo1=NULL, *fo2=NULL;

    sprintf(ftmp1, "%s_temp1", inpfile);
    sprintf(ftmp2, "%s_temp2", inpfile);
    
    fo1=fopen(ftmp1,"w");
    fo2=fopen(ftmp2,"w");

    line=get_lines_from_file(inpfile, &nline);

    for(i=0; i<nline; i++){
        if(strstr(line[i],"loop_") && strstr(line[i+1],"_refln.")){
            for(j=i; j<nline; j++){
                fprintf(fo2,"%s",line[j]);
            }
            break;
        }
        fprintf(fo1,"%s",line[i]);
        n1++;
    }
    fclose(fo1); fclose(fo2);
    if(n1>2) cifparse(ftmp1, ftmp1, 80);
    
    sprintf(command, "/bin/csh -c \" /bin/cat  %s  %s  > %s\" ", ftmp1, ftmp2, outfile);
    int fake;
    fake = system(command);
    fake++;
    if (VERB==0)delete_file(ftmp1);
    if (VERB==0)delete_file(ftmp2);
     
    if(nline) free_memory_2d(line, nline);
    

}


void clean_up_sf(char *ifile, char *ofile)
//correct the cif tokens, if the ciflist is inside of the SF file.
{
    char  token1[256], token2[256], token[256], token_change[5200];
    char  **line, **refln_list, tmp[1024], str[2560];
    int  i=0, j, n=0, n1=0, nline, n2=0, n3=0, nlist=0;
    FILE *fp;

//the list below are the wrong cif token (left) in the mmcif file
    static const char *ciflist[] = 
        {
" _refln.A_calc_au                          _refln.A_calc ",
" _refln.B_calc_au                          _refln.B_calc ",
" _refln.A_cal                              _refln.A_calc ",
" _refln.B_cal                              _refln.B_calc ",
" _refln.resolution                         _refln.d_spacing ",
" _refln.Fc_au                              _refln.F_calc_au ",
" _refln.F_sigma                            _refln.F_meas_sigma ",
" _refln.F_meas_au_sigma                    _refln.F_meas_sigma_au ",
" _refln.F_meas_siygma_au                   _refln.F_meas_sigma_au ",
" _refln.F_squared_meas_sigma               _refln.F_squared_sigma ",
" _refln.I_meas_au                          _refln.intensity_meas ",
" _refln.intensity                          _refln.intensity_meas ",
" _refln.intensity_meas_au                  _refln.intensity_meas ",
" _refln.intensity_sigm                     _refln.intensity_sigma ",
" _refln.I_meas_sigma_au                    _refln.intensity_sigma ",
" _refln.intensity_sigma_au                 _refln.intensity_sigma ",
" _refln.intensity_meas_sigma               _refln.intensity_sigma ",
" _refln.intensity_meas_sigma_au            _refln.intensity_sigma ",
" _refln.dfano                              _refln.pdbx_anom_difference ",
" _refln.DF_anomalous                       _refln.pdbx_anom_difference ",
" _refln.ccp4_phase_anom                    _refln.pdbx_anom_difference ",
" _refln.pdbx_phase_anom                    _refln.pdbx_anom_difference ",
" _refln.ccp4_anomalous_diff                _refln.pdbx_anom_difference ",
" _refln.ccp4_SAD_phase_anom                _refln.pdbx_anom_difference ",
" _refln.pdbx_anomalous_diff                _refln.pdbx_anom_difference ",
" _refln.pdbx_SAD_phase_anom                _refln.pdbx_anom_difference ",
" _refln.pdbx_anomalous_meas_au             _refln.pdbx_anom_difference ",
" _refln.I_ano_meas                         _refln.pdbx_anom_difference ",
" _refln.I_ano_sigma                        _refln.pdbx_anom_difference_sigma ",
" _refln.DF_anomalous_sigma                 _refln.pdbx_anom_difference_sigma ",
" _refln.ccp4_phase_anom_sigma              _refln.pdbx_anom_difference_sigma ",
" _refln.pdbx_phase_anom_sigma              _refln.pdbx_anom_difference_sigma ",
" _refln.ccp4_anomalous_diff_sigma          _refln.pdbx_anom_difference_sigma ",
" _refln.ccp4_SAD_phase_anom_sigma          _refln.pdbx_anom_difference_sigma ",
" _refln.pdbx_SAD_phase_anom_sigma          _refln.pdbx_anom_difference_sigma ",
" _refln.pdbx_anomalous_meas_sigma_au       _refln.pdbx_anom_difference_sigma ",
" _refln.F-_meas_au                         _refln.pdbx_F_minus ",
" _refln.pdbx_F_meas_minus                  _refln.pdbx_F_minus ",
" _refln.ccp4_F_meas_minus_au               _refln.pdbx_F_minus ",
" _refln.pdbx_F_meas_minus_au               _refln.pdbx_F_minus ",
" _refln.ccp4_SAD_F_meas_minus_au           _refln.pdbx_F_minus ",
" _refln.F-_meas_sigma_au                   _refln.pdbx_F_minus_sigma ",
" _refln.pdbx_F_meas_minus_sigma            _refln.pdbx_F_minus_sigma  ",
" _refln.ccp4_F_meas_minus_sigma_au         _refln.pdbx_F_minus_sigma ",
" _refln.pdbx_F_meas_minus_sigma_au         _refln.pdbx_F_minus_sigma ",
" _refln.ccp4_SAD_F_meas_minus_sigma_au     _refln.pdbx_F_minus_sigma ",
" _refln.F+_meas_au                         _refln.pdbx_F_plus ",
" _refln.pdbx_F_meas_plus                   _refln.pdbx_F_plus ",
" _refln.ccp4_F_meas_plus_au                _refln.pdbx_F_plus ",
" _refln.pdbx_F_meas_plus_au                _refln.pdbx_F_plus ",
" _refln.ccp4_SAD_F_meas_plus_au            _refln.pdbx_F_plus ",
" _refln.F+_meas_sigma_au                   _refln.pdbx_F_plus_sigma ",
" _refln.pdbx_F_meas_plus_sigma             _refln.pdbx_F_plus_sigma ",
" _refln.ccp4_F_meas_plus_sigma_au          _refln.pdbx_F_plus_sigma ",
" _refln.pdbx_F_meas_plus_sigma_au          _refln.pdbx_F_plus_sigma ",
" _refln.ccp4_SAD_F_meas_plus_sigma_au      _refln.pdbx_F_plus_sigma ",
" _refln.pdbx_HLA                           _refln.pdbx_HL_A_iso ",
" _refln.ccp4_HL_A_iso                      _refln.pdbx_HL_A_iso ",
" _refln.ccp4_SAD_HL_A_iso                  _refln.pdbx_HL_A_iso ",
" _refln.pdbx_SAD_HL_A_iso                  _refln.pdbx_HL_A_iso ",
" _refln.pdbx_HLB                           _refln.pdbx_HL_B_iso ",
" _refln.ccp4_HL_B_iso                      _refln.pdbx_HL_B_iso ",
" _refln.ccp4_SAD_HL_B_iso                  _refln.pdbx_HL_B_iso ",
" _refln.pdbx_SAD_HL_B_iso                  _refln.pdbx_HL_B_iso ",
" _refln.pdbx_HLC                           _refln.pdbx_HL_C_iso ",
" _refln.ccp4_HL_C_iso                      _refln.pdbx_HL_C_iso ",
" _refln.ccp4_SAD_HL_C_iso                  _refln.pdbx_HL_C_iso ",
" _refln.pdbx_SAD_HL_C_iso                  _refln.pdbx_HL_C_iso ",
" _refln.pdbx_HLD                           _refln.pdbx_HL_D_iso ",
" _refln.ccp4_HL_D_iso                      _refln.pdbx_HL_D_iso ",
" _refln.ccp4_SAD_HL_D_iso                  _refln.pdbx_HL_D_iso ",
" _refln.pdbx_SAD_HL_D_iso                  _refln.pdbx_HL_D_iso ",
" _refln.ccp4_I_minus                       _refln.pdbx_I_minus ",
" _refln.ccp4_I_minus_sigma                 _refln.pdbx_I_minus_sigma ",
" _refln.ccp4_I_plus                        _refln.pdbx_I_plus ",
" _refln.ccp4_I_plus_sigma                  _refln.pdbx_I_plus_sigma ",
" _refln.pahse_calc                         _refln.phase_calc ",
" _refln.F.phase_calc                       _refln.phase_calc ",
" _refln.phase_au                           _refln.phase_meas ",
" _refln.pdbx_fom_weighted_fmap             _refln.pdbx_FWT ",
" _refln.statu                              _refln.status ",
" _refln.F_status                           _refln.status ",
" _refln.status_au                          _refln.status ",
" _refln.R_free_flag                        _refln.status ",
" _refln.observed_status                    _refln.status ",
" _refln.waveLEngth_id                      _refln.wavelength_id ",
" _refln.wavelength_di                      _refln.wavelength_id ",
" _refln.index_l>                           _refln.index_l ",
" _refln.fiber_coordinate                  _refln.pdbx_fiber_coordinate ",
" _refln.fiber_F_meas_au                   _refln.pdbx_fiber_F_meas_au ",
" _refln.fiber_layer                       _refln.pdbx_fiber_layer ",
" _refln_index_h                            _refln.index_h ",
" _refln_index_k                            _refln.index_k ",
" _refln_index_l                            _refln.index_l ",

" _refln_observed_status                    _refln.status ",
" _refln_F_squared_meas                     _refln.F_squared_meas ",
" _refln_F_squared_calc                     _refln.F_squared_calc ",
" _refln_phase_calc                         _refln.phase_calc ",
" _refln_d_spacing                          _refln.d_spacing ",
" _gsas_i100_meas                           _refln.pdbx_gsas_i100_meas ",
 
" _pd_meas_intensity_total            _pdbx_powder_data.pd_meas_intensity_total",
" _pd_proc_intensity_total            _pdbx_powder_data.pd_proc_intensity_total",
" _pd_proc_ls_weight                  _pdbx_powder_data.pd_proc_ls_weight",
" _pd_proc_intensity_bkg_calc         _pdbx_powder_data.pd_proc_intensity_bkg_calc",
" _pd_calc_intensity_total            _pdbx_powder_data.pd_calc_intensity_total",


" _refln_F_squared_sigma                    _refln.F_squared_sigma ",
" _refln_F_calc                             _refln.F_calc ",
" _diffrn_radiation.pdbx_wavelength         _diffrn_radiation_wavelength.wavelength ",
" _diffrn_radiation.diffrn_id               _diffrn_radiation_wavelength.id ",
"END",


"_refln.F_meas  _refln.intensity_meas ",
"_refln.F_meas_sigma  _refln.intensity_sigma",
"_refln.F_meas_au  _refln.intensity_meas ",
"_refln.F_meas_sigma_au  _refln.intensity_sigma",
"_refln.intensity_meas  _refln.F_meas_au",
"_refln.intensity_sigma  _refln.F_meas_sigma_au",

"END"
        };

    
    refln_list=set_memory_2d(200, 90);
    
    line=get_lines_from_file(ifile, &nline);
    if( (fp = fopen(ofile, "w")) ==NULL || nline<10 ){        
        printf("Error! Can not open the file (%s) or no written permission! \n",ifile);
        return ;
    }
    
    strcpy(token_change,"");
    for(i=0; i<nline; i++){
        strcpy(str, line[i]);
        
        if(strstr_case(str, "data_")){
            if(strchr(str, '/')) replace_char(str, '/', '_');
            n2=0;

            
        }else if(strstr_case(str,"_symmetry_equiv.") ){
            if(i>=nline-1) continue;
            sscanf(line[i+1], "%s", tmp);
            if(strstr(tmp, "_atom_sites.")){
                printf("Error! Table 'symmetry_equiv' is empty (%s)\n", ifile);
                
                fprintf(fp, "%s 1 \n", str);
                continue;
                
            }
            
        }else if(strstr_case(str,"_pdbx_powder_refln.")){// powder
            replace_first_str(str, "_pdbx_powder_refln.", "_refln.", tmp);
            if (strstr(tmp, "pd_calc_intensity_total_"))
                replace_first_str(tmp, "pd_calc_intensity_total_", "pd_calc_intensity_total", tmp);
            fprintf(fp, "%s", tmp);
            continue;
            
            
        }else if(strstr_case(str,"_diffrn_radiation.")){// do not use this
            if(strstr(str,"_diffrn_radiation.pdbx_wavelength")){
                replace_first_str(str, "_diffrn_radiation.pdbx_wavelength",
                                  "_diffrn_radiation_wavelength.wavelength", tmp);
            }else{
                replace_first_str(str, "_diffrn_radiation.", "_diffrn_radiation_wavelength.", tmp);
            }
            
            
            fprintf(fp, "%s", tmp);
            continue;
            
        }else if(strstr_case(str,"_refln.") ){//current cif parser failed to parse loop
            n1=sscanf(str,"%s %d", tmp, &n);
            rid_of_front_end_space(tmp);
            
            if(n1 == 2){
                if (!strncmp_case(tmp,"_refln.", 7) && n>0) {
                    printf("Error in cif! deleting line %s\n", str);
                    continue;
                }
                
            }else if(n1 == 1  && !strncmp(tmp,"_refln.", 7) ){
                compare_refln(ifile, tmp);
                
                
                n2++;
                if(n2 == 1) get_refln_list(i, nline, line, refln_list, &nlist);
                               
                sscanf(str, "%s", token1);
                rid_of_front_end_space(token1);
               
                for(j=0; j<500; j++){
                    if(!strcmp_case(ciflist[j], "END"))break;
                    sscanf(ciflist[j], "%s %s", token2, token);
                    if(!strcmp_case(token1, token2)){ //wrong token
                        n3=check_refln_list(token, nlist, refln_list);
                        if(n3==0){ //token not in the refln_list
                            sprintf(tmp,"%s replaced by %s;", token2, token);
                            printf("Warning (%s): %s\n", ifile, tmp);
                            strcat(token_change,tmp);
                            replace_first_str(str,token2, token, str);
                        }else{
                            printf("Error! The item %s can not be replaced by %s\n", token2, token);
                        }
                        
                        break;
                    }
                }
            }
            
        }else{
                sscanf(str, "%s", token1);
                rid_of_front_end_space(token1);

                for(j=0; j<500; j++){
                    if(!strcmp_case(ciflist[j], "END"))break;
                    sscanf(ciflist[j], "%s %s", token2, token);
                    if(!strcmp_case(token1, token2)){ //wrong token
                        n3=check_refln_list(token, nlist, refln_list);
                        if(n3==0){ //token not in the refln_list
                            sprintf(tmp,"%s replaced by %s;", token2, token);
                            printf("Warning (%s): %s\n", ifile, tmp);
                            strcat(token_change,tmp);
                            replace_first_str(str,token2, token, str);
                        }else{
                            printf("Error! The item %s can not be replaced by %s\n", token2, token);
                        }

                        break;
                    }
                }
        }




        fprintf(fp, "%s", str);
    }
    if(nline>0) free_memory_2d(line, nline);
    fclose(fp);
    
    
}

void get_refln_list(int start, int nline, char **line, char **refln_list, int *nlist)
//get the list of _refln. to see if the replaced item is in the list

{
    char str[512];
    int i, n=0, m=0;

    for(i=start; i<nline; i++){
        if(n++> 50) break;
        strcpy(str, line[i]);
        rid_of_front_end_space(str);
        if(!strncmp(str,"_refln.", 7)){
            strcpy(refln_list[m], str);
            m++;
        }
    }
    *nlist= m;
}

void get_mmcif_item_index(int n, char **item, int index[],  int *nitem_tot)  
//nitem=19, it should change when items change. 

{
    int i;

    *nitem_tot=19;
    
    for(i=0; i<n; i++){
        if(!strcmp_case(item[i], "_refln.index_h")){
            index[0]=i;
        }else if(!strcmp_case(item[i], "_refln.index_k")){
            index[1]=i;
        }else if(!strcmp_case(item[i], "_refln.index_l")){
            index[2]=i;
        }else if(!strcmp_case(item[i], "_refln.F_meas_au")){
            index[3]=i;
        }else if(!strcmp_case(item[i], "_refln.F_meas_sigma_au")){
            index[4]=i;
        }else if(!strcmp_case(item[i], "_refln.intensity_meas")){
            index[5]=i;
        }else if(!strcmp_case(item[i], "_refln.intensity_sigma")){
            index[6]=i;
        }else if(!strcmp_case(item[i], "_refln.F_squared_meas")){
            index[7]=i;
        }else if(!strcmp_case(item[i], "_refln.F_squared_sigma")){
            index[8]=i;
        }else if(!strcmp_case(item[i], "_refln.status")){
            index[9]=i;
        }else if(!strcmp_case(item[i], "_refln.F_meas")){
            index[10]=i;
        }else if(!strcmp_case(item[i], "_refln.F_meas_sigma")){
            index[11]=i;
        }else if(!strcmp_case(item[i], "_refln.F_calc")){
            index[12]=i;
        }else if(!strcmp_case(item[i], "_refln.F_calc_au")){
            index[13]=i;
        }else if(!strcmp_case(item[i], "_refln.F_squared_calc")){
            index[14]=i;
        }else if(!strcmp_case(item[i], "_refln.intensity_calc")){
            index[15]=i;
        }else if(!strcmp_case(item[i], "_refln.fom")){
            index[16]=i;
        }else if(!strcmp_case(item[i], "_refln.wavelength_id")){
            index[17]=i;
        }else if(!strcmp_case(item[i], "_refln.crystal_id")){
            index[18]=i;
        }
        
    }
    
    *nitem_tot=19;
    
        /*
    for(i=0; i<n; i++){
        if(index[i]>=0)printf("cif-items %d  %d (%d) %s\n",
        n, i, index[i],item[index[i]]);
        printf("cif-items %d  %d (%d)\n",n, i, index[i]);
    }
        */    
}


void check_data_block_name(char *file, char *block_id, char *pdbid, int num)
// num=0 is the first data block
{
    int n=0, prob=0;


    rid_of_front_end_space(block_id);
    if(strchr(block_id, '/')) replace_char(block_id, '/', '_');
    if(strchr(block_id, ')')) replace_char(block_id, ')', '_');
    if(strchr(block_id, '(')) replace_char(block_id, '(', '_');

    n=strlen(block_id);
    pdbid[0] = '\0';
    
    if (num==0){
        if (n != 7 || block_id[0] !='r' || !strstr(block_id +5, "sf")){
            prob=1;
        }else{
            strncpy(pdbid, block_id + 1, 4);
            pdbid[4]='\0';
        }

    }else {
        if (n != 8 || block_id[0] !='r' || !strstr(block_id +6, "sf")){
            prob=1;
        }else if(num<25 && (block_id[5]<65 || block_id[5]>90)){
            prob=1;
        }else{
            strncpy(pdbid, block_id + 1, 4);
            pdbid[4]='\0';
        }
    }

    if (prob==1){
        sprintf(INFO, "Note: File has non-standard blockID (block_number=%d: block_id=%s).", num+1, block_id);
        pinfo(INFO,0);
    }

    rid_of_space(pdbid);

}

